/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.binaryfile;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.*;

import java.io.IOException;
import java.io.InputStream;
import java.util.List;
import java.util.concurrent.Future;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.hateoas.CollectionModel;
import org.springframework.hateoas.EntityModel;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestPart;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.method.annotation.StreamingResponseBody;

import de.itvsh.kop.common.errorhandling.TechnicalException;
import de.ozgcloud.alfa.common.file.OzgFile;

@RestController
@RequestMapping(BinaryFileController.PATH)
public class BinaryFileController {

	static final String PATH = "/api/binaryFiles"; // NOSONAR

	@Autowired
	private BinaryFileService service;
	@Autowired
	private BinaryFileModelAssembler modelAssembler;
	@Autowired
	private FileIdMapper fileIdMapper;

	@GetMapping("/{fileId}")
	public EntityModel<OzgFile> getFile(@PathVariable FileId fileId) {
		return modelAssembler.toModel(service.getFile(fileId));
	}

	@PostMapping("/{vorgangId}/{field}/file")
	public Future<ResponseEntity<Void>> uploadFile(@PathVariable String vorgangId, @PathVariable String field, @RequestPart MultipartFile file) {
		var fileIdFuture = service.uploadFile(buildBinaryFileUploadRequest(vorgangId, field, file));

		return fileIdFuture.thenApply(fileId -> ResponseEntity.created(linkTo(BinaryFileController.class).slash(fileId).toUri()).build());
	}

	UploadBinaryFileRequest buildBinaryFileUploadRequest(String vorgangId, String field, MultipartFile file) {
		return UploadBinaryFileRequest.builder()
				.vorgangId(vorgangId)
				.field(field)
				.uploadStream(getInputStream(file))
				.name(file.getOriginalFilename())
				.contentType(file.getContentType())
				.size(file.getSize())
				.build();
	}

	InputStream getInputStream(MultipartFile file) {
		try {
			return file.getInputStream();
		} catch (IOException e) {
			throw new TechnicalException("Error reading File data as stream.", e);
		}
	}

	public CollectionModel<EntityModel<OzgFile>> getFiles(List<FileId> fileIds) {
		var files = service.getFiles(mapToFileId(fileIds)).toList();

		return modelAssembler.toCollectionModel(files);
	}

	private List<FileId> mapToFileId(List<FileId> fileIds) {
		return fileIds.stream()
				.map(fileIdMapper::toString)
				.map(FileId::from)
				.toList();
	}

	@PreAuthorize("@downloadAuthenticationHandler.canDownloadFile(#fileId)")
	@GetMapping(value = "/{fileId}", produces = { MediaType.APPLICATION_OCTET_STREAM_VALUE,
			"images/*" }, consumes = MediaType.APPLICATION_OCTET_STREAM_VALUE)
	public ResponseEntity<StreamingResponseBody> getFileData(@PathVariable FileId fileId) {
		var ozgFile = service.getFile(fileId);

		return buildResponseEntity(ozgFile, createDownloadStreamingBody(fileId));
	}

	StreamingResponseBody createDownloadStreamingBody(FileId fileId) {
		return out -> service.writeFileContent(fileId, out);
	}

	ResponseEntity<StreamingResponseBody> buildResponseEntity(OzgFile ozgFile, StreamingResponseBody responseBody) {
		return ResponseEntity.ok()
				.header(HttpHeaders.CONTENT_DISPOSITION, String.format("attachment; filename=%s", ozgFile.getName()))
				.contentLength(ozgFile.getSize())
				.contentType(MediaType.valueOf(ozgFile.getContentType()))
				.body(responseBody);
	}
}