/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.command;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Objects;
import java.util.Optional;
import java.util.function.Predicate;
import java.util.stream.Collectors;

import org.apache.commons.beanutils.BeanMap;
import org.apache.commons.lang3.StringUtils;
import org.mapstruct.Mapper;

import de.itvsh.ozg.pluto.grpc.command.GrpcCommandBody;
import de.itvsh.ozg.pluto.grpc.command.GrpcCommandBodyField;

@Mapper
public interface CommandBodyMapper {

	final Predicate<Entry<Object, Object>> IS_NOT_CLASS_VALUE = entry -> !StringUtils.equals("class", entry.getKey().toString());
	final Predicate<Entry<Object, Object>> IS_NOT_VERSION_VALUE = entry -> !StringUtils.equals("version", entry.getKey().toString());

	static final String VORGANG_ID_PROPERTY = "vorgangId";
	static final String ITEM_NAME_PROPERTY = "itemName";
	static final String ITEM_PROPERTY = "item";

	default GrpcCommandBody mapToBody(Map<String, String> bodyMap) {
		if (Objects.isNull(bodyMap)) {
			return GrpcCommandBody.getDefaultInstance();
		}

		var builder = GrpcCommandBody.newBuilder();

		bodyMap.entrySet().stream()
				.filter(entry -> Objects.nonNull(entry.getValue()))
				.map(entry -> GrpcCommandBodyField.newBuilder().setName(entry.getKey()).setValue(entry.getValue()).build())
				.forEach(builder::addField);

		return builder.build();
	}

	default List<GrpcCommandBodyField> mapToBodyFields(CommandBody body) {
		if (Objects.isNull(body)) {
			return Collections.emptyList();
		}

		return mapToBodyFields(new BeanMap(body));
	}

	default List<GrpcCommandBodyField> mapToBodyFields(Map<Object, Object> bodyMap) {
		if (Objects.isNull(bodyMap)) {
			return Collections.emptyList();
		}

		return bodyMap.entrySet().stream()
				.filter(IS_NOT_CLASS_VALUE)
				.map(this::buildBodyField)
				.toList();
	}

	default GrpcCommandBodyField buildBodyField(Entry<Object, Object> entry) {
		var bodyFieldMapper = GrpcCommandBodyField.newBuilder().setName(entry.getKey().toString());

		if (Objects.nonNull(entry.getValue())) {
			bodyFieldMapper.setValue(getNullableStringValue(entry.getValue()));
		}
		return bodyFieldMapper.build();
	}

	default String getNullableStringValue(Object entryValue) {
		return Optional.ofNullable(entryValue).map(Object::toString).orElse(null);
	}

	default Map<String, String> map(GrpcCommandBody body) {
		return body.getFieldList().stream().collect(Collectors.toMap(GrpcCommandBodyField::getName, GrpcCommandBodyField::getValue));
	}

	default Map<String, Object> mapToBodyMap(CreateCommand command, String itemName) {
		return Map.of(
				VORGANG_ID_PROPERTY, command.getVorgangId(),
				ITEM_NAME_PROPERTY, itemName,
				ITEM_PROPERTY, fromObjectToMap(command.getBody()));
	}

	@SuppressWarnings("unchecked")
	public default Map<String, Object> fromObjectToMap(Object object) {
		if (object instanceof Map<?, ?> objMap) {
			return (Map<String, Object>) objMap;
		}

		return new BeanMap(object).entrySet().stream()
				.filter(IS_NOT_CLASS_VALUE)
				.filter(IS_NOT_VERSION_VALUE)
				.collect(HashMap::new, this::addToMap, Map::putAll);
	}

	default void addToMap(Map<String, Object> map, Entry<Object, Object> entry) {
		map.put(entry.getKey().toString(), getValue(entry.getValue()));
	}

	default Object getValue(Object entryValue) {
		return checkEnumValue(entryValue);
	}

	private Object checkEnumValue(Object entryValue) {
		if (entryValue instanceof Enum<?> enumValue) {
			return enumValue.name();
		}
		return entryValue;
	}
}