/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.user;

import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.util.function.Predicate;

import org.apache.commons.lang3.StringUtils;
import org.springframework.security.authentication.AuthenticationTrustResolver;
import org.springframework.security.authentication.AuthenticationTrustResolverImpl;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.oauth2.jwt.Jwt;

import lombok.AccessLevel;
import lombok.NoArgsConstructor;

@NoArgsConstructor(access = AccessLevel.PRIVATE)
public class CurrentUserHelper {

	static final String ROLE_PREFIX = "ROLE_";
	private static final String SUB_CLAIM_KEY = "sub";

	private static final AuthenticationTrustResolver TRUST_RESOLVER = new AuthenticationTrustResolverImpl();
	private static final Predicate<Authentication> IS_TRUSTED = auth -> !TRUST_RESOLVER.isAnonymous(auth);
	private static final Predicate<String> IS_ROLE_PREFIX_MISSING = role -> !role.startsWith(ROLE_PREFIX);

	public static boolean hasRole(String role) {
		var auth = getAuthentication();

		if ((Objects.isNull(auth)) || (Objects.isNull(auth.getPrincipal()))) {
			return false;
		}
		return containsRole(auth.getAuthorities(), role);
	}

	public static boolean containsRole(Collection<? extends GrantedAuthority> authorities, String role) {
		if (Objects.isNull(authorities)) {
			return false;
		}
		return containsRole(prepareRoleForCheck(role), authorities);
	}

	static boolean containsRole(String role, Collection<? extends GrantedAuthority> authorities) {
		return authorities.stream().anyMatch(a -> isAuthorityEquals(role, a.getAuthority()));
	}

	private static boolean isAuthorityEquals(String role, String authority) {
		return StringUtils.equalsIgnoreCase(role, authority) || StringUtils.equalsIgnoreCase(prepareRoleForCheck(role), authority);
	}

	static String prepareRoleForCheck(String roleToCheck) {
		return Optional.ofNullable(roleToCheck)
				.filter(IS_ROLE_PREFIX_MISSING)
				.map(role -> String.format("%s%s", ROLE_PREFIX, role))
				.orElse(roleToCheck);
	}

	public static UserId getCurrentUserId() {
		return UserId.from(getSubClaim());
	}

	private static String getSubClaim() {
		return ((Jwt) getAuthentication().getPrincipal()).getClaim(SUB_CLAIM_KEY);
	}

	public static Authentication getAuthentication() {
		return findAuthentication().orElseThrow(() -> new IllegalStateException("No authenticated User found"));
	}

	private static Optional<Authentication> findAuthentication() {
		return Optional.ofNullable(SecurityContextHolder.getContext().getAuthentication()).filter(IS_TRUSTED);
	}
}
