/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.user;

import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.hierarchicalroles.RoleHierarchy;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.oauth2.jwt.Jwt;
import org.springframework.stereotype.Service;

import de.itvsh.kop.common.errorhandling.TechnicalException;
import de.ozgcloud.alfa.common.binaryfile.GoofyUserWithFileId;

@Service
public class CurrentUserService {

	public static final String VERIFY_HAS_ROLE_VERWALTUNG_USER = "@currentUserService.hasRole(\"" + UserRole.VERWALTUNG_USER + "\")";
	public static final String VERIFY_HAS_ROLE_VERWALTUNG_LOESCHEN = "@currentUserService.hasRole(\"" + UserRole.VERWALTUNG_LOESCHEN + "\")";
	public static final String ATTRIBUTE_NAME_USER_ID = "ozgCloudUserId";

	static final String USER_ATTRIBUTE_ORGANISATIONSEINHEIT_ID = "organisationseinheitId";

	static final String KEYCLOAK_USER_PREFERRED_USERNAME = "preferred_username";
	static final String KEYCLOAK_USER_GIVEN_NAME = "given_name";
	static final String KEYCLOAK_USER_FAMILY_NAME = "family_name";

	@Autowired
	private UserService userService;
	@Autowired
	private RoleHierarchy roleHierarchy;

	public boolean hasRole(String role) {
		return CurrentUserHelper.hasRole(role) || hasRoleReachable(role);
	}

	private boolean hasRoleReachable(String role) {
		var reachableRoles = roleHierarchy.getReachableGrantedAuthorities(getAuthorities());

		return CurrentUserHelper.containsRole(reachableRoles, role);
	}

	public UserProfile getUser() {
		return getDownloadUser().orElseGet(this::buildUserProfile);
	}

	private UserProfile buildUserProfile() {
		var userBuilder = UserProfile.builder()
				.id(getUserId())
				.authorities(getAuthorities());

		getCurrentSecurityToken().ifPresent(token -> userBuilder
				.userName(token.getClaimAsString(KEYCLOAK_USER_PREFERRED_USERNAME))
				.firstName(token.getClaimAsString(KEYCLOAK_USER_GIVEN_NAME))
				.lastName(token.getClaimAsString(KEYCLOAK_USER_FAMILY_NAME))
				.organisationseinheitIds(getOrganisationsEinheitIds(token)));

		return userBuilder.build();
	}

	public Collection<GrantedAuthority> getAuthorities() {
		return Collections.unmodifiableCollection(CurrentUserHelper.getAuthentication().getAuthorities());
	}

	public UserId getUserId() {
		return findUserId()
				.orElseThrow(() -> new TechnicalException("Cannot find internal UserId. Check sync with UserManager or Token Mapper in keycloak."));
	}

	public Optional<UserId> findUserId() {
		return Optional.ofNullable(getSingleClaimValue(ATTRIBUTE_NAME_USER_ID)
				.map(UserId::from)
				.orElseGet(this::getInternalId))
				.filter(Objects::nonNull);
	}

	private UserId getInternalId() {
		return userService.getInternalId(CurrentUserHelper.getCurrentUserId()).orElse(null);
	}

	List<String> getOrganisationsEinheitIds(Jwt jwt) {
		return Optional.ofNullable(jwt)
				.map(token -> token.getClaim(USER_ATTRIBUTE_ORGANISATIONSEINHEIT_ID))
				.map(col -> (Collection<?>) col).orElse(Collections.emptyList()) // NOSONAR - Collection.class::cast has type-safty issue
				.stream().map(Object::toString).toList();
	}

	private Optional<UserProfile> getDownloadUser() {
		return Optional.of(CurrentUserHelper.getAuthentication().getPrincipal())
				.filter(GoofyUserWithFileId.class::isInstance)
				.map(GoofyUserWithFileId.class::cast)
				.map(GoofyUserWithFileId::getUser);
	}

	Optional<String> getSingleClaimValue(String attributeName) {
		return getCurrentSecurityToken()
				.map(token -> token.getClaim(attributeName))
				.map(String.class::cast);
	}

	Optional<Jwt> getCurrentSecurityToken() {
		var principal = CurrentUserHelper.getAuthentication().getPrincipal();

		if (principal instanceof Jwt kcPrincipal) {
			return Optional.of(kcPrincipal);
		}

		return Optional.empty();
	}
}