package de.ozgcloud.alfa.loeschanforderung;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.*;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.hateoas.EntityModel;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.PatchMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import de.ozgcloud.alfa.common.attacheditem.VorgangAttachedItemService;
import de.ozgcloud.alfa.common.command.Command;
import de.ozgcloud.alfa.common.command.CommandController;
import de.ozgcloud.alfa.common.command.CommandOrder;
import de.ozgcloud.alfa.common.command.CommandStatus;
import de.ozgcloud.alfa.common.command.CreateCommand;
import de.ozgcloud.alfa.common.command.StatusPatch;
import de.ozgcloud.alfa.common.errorhandling.OrderNotAllowedException;
import de.ozgcloud.alfa.vorgang.VorgangController;

@RestController
@RequestMapping(LoeschAnforderungCommandController.BASE_PATH)
public class LoeschAnforderungCommandController {

	static final String BASE_PATH = "/api/loeschanforderungs/{loeschAnforderungId}/commands"; // NOSONAR

	@Autowired
	private LoeschAnforderungService loeschAnforderungService;

	@Autowired
	private VorgangAttachedItemService vorgangAttachedItemService;

	@Autowired
	private VorgangController vorgangController;

	@Autowired
	private CommandController commandController;

	@PreAuthorize("@loeschAnforderungPreAuthorizeHandler.hasPermissionForOrder(#loeschAnforderungId, #command.order)")
	@PostMapping
	public ResponseEntity<Void> createCommand(@PathVariable String loeschAnforderungId, @RequestBody CreateCommand command) {
		var createdCommand = executeCommand(command, loeschAnforderungId);

		return ResponseEntity.created(linkTo(CommandController.class).slash(createdCommand.getId()).toUri()).build();
	}

	private Command executeCommand(CreateCommand command, String loeschAnforderungId) {
		return switch (command.getOrder()) {
			case VORGANG_LOESCHEN -> loeschAnforderungService.vorgangLoeschen(command, loeschAnforderungId);
			case LOESCH_ANFORDERUNG_ZURUECKNEHMEN -> loeschAnforderungZuruecknehmen(loeschAnforderungId);
			default -> throw new OrderNotAllowedException(command.getOrder());
		};
	}

	private Command loeschAnforderungZuruecknehmen(String loeschAnforderungId) {
		var vorgangAttachedItem = vorgangAttachedItemService.getById(loeschAnforderungId);
		var vorgang = vorgangController.getVorgang(vorgangAttachedItem.getVorgangId());
		return loeschAnforderungService.zuruecknehmen(vorgangAttachedItem, vorgang);
	}

	@PatchMapping("{commandId}")
	public ResponseEntity<EntityModel<Command>> revoke(@PathVariable String loeschAnforderungId, @PathVariable String commandId,
			@RequestBody StatusPatch patch) {
		if (patch.getStatus() == CommandStatus.REVOKED) {
			var command = commandController.getById(commandId).getContent();

			if (command.getOrder() == CommandOrder.VORGANG_ZUM_LOESCHEN_MARKIEREN) {
				return handleVorgangZumLoeschenMarkierenRevokeCommand(loeschAnforderungId, command, patch);
			}
			if (command.getOrder() == CommandOrder.LOESCH_ANFORDERUNG_ZURUECKNEHMEN) {
				return handleLoeschAnforderungZuruecknehmenRevokeCommand(command, patch);
			}
		}
		return ResponseEntity.status(HttpStatus.FORBIDDEN).build();
	}

	ResponseEntity<EntityModel<Command>> handleVorgangZumLoeschenMarkierenRevokeCommand(String loeschAnforderungId, Command command,
			StatusPatch patch) {
		loeschAnforderungService.deleteLoeschAnforderung(loeschAnforderungId);

		return commandController.revoke(command.getId(), patch);
	}

	ResponseEntity<EntityModel<Command>> handleLoeschAnforderungZuruecknehmenRevokeCommand(Command command, StatusPatch patch) {
		loeschAnforderungService.reCreateLoeschAnforderung(command);

		var changeStatusCommandId = loeschAnforderungService.getChangeStatusCommandId(command);
		return commandController.revoke(changeStatusCommandId, patch);
	}
}