/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.vorgang;

import static java.util.Optional.*;
import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.*;

import java.util.Objects;
import java.util.Optional;
import java.util.function.Predicate;

import org.apache.commons.lang3.ObjectUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.Link;
import org.springframework.hateoas.LinkRelation;
import org.springframework.hateoas.server.RepresentationModelProcessor;
import org.springframework.stereotype.Component;

import de.ozgcloud.alfa.attachment.AttachmentController;
import de.ozgcloud.alfa.common.FeatureToggleProperties;
import de.ozgcloud.alfa.common.ModelBuilderFactory;
import de.ozgcloud.alfa.common.command.CommandController.CommandByRelationController;
import de.ozgcloud.alfa.common.user.CurrentUserService;
import de.ozgcloud.alfa.common.user.UserManagerUrlProvider;
import de.ozgcloud.alfa.common.user.UserRole;
import de.ozgcloud.alfa.historie.HistorieController;
import de.ozgcloud.alfa.kommentar.KommentarController.KommentarByVorgangController;
import de.ozgcloud.alfa.postfach.PostfachMailController;
import de.ozgcloud.alfa.representation.RepresentationController;
import de.ozgcloud.alfa.vorgang.VorgangProperties.VorgangProperty;
import de.ozgcloud.alfa.vorgang.forwarding.ForwardingController;

@Component
class VorgangWithEingangProzessor implements RepresentationModelProcessor<EntityModel<VorgangWithEingang>> {

	static final LinkRelation REL_KOMMENTARE = LinkRelation.of("kommentare");
	static final LinkRelation REL_ATTACHMENTS = LinkRelation.of("attachments");
	static final LinkRelation REL_REPRESENTATIONS = LinkRelation.of("representations");
	static final LinkRelation REL_POSTFACH_MAILS = LinkRelation.of("postfachMails");
	static final LinkRelation REL_VORGANG_FORWARDING = LinkRelation.of("forwarding");
	static final LinkRelation REL_HISTORIE = LinkRelation.of("historie");
	static final LinkRelation REL_BESCHEID = LinkRelation.of("createBescheid");

	static final String REL_SEARCH_USER = "search-user-profiles";
	static final String USER_PROFILE_SEARCH_DELETED_PARAM = "deleted";
	static final String USER_PROFILE_SEARCH_ORGANISATIONS_EINHEIT_ID_PARAM = "organisationsEinheitId";

	private static final Predicate<EingangHeader> HAS_FORM_ID_WITH_ENGINE_NAME = eingangHeader -> ObjectUtils
			.allNotNull(eingangHeader.getFormEngineName(), eingangHeader.getFormId());

	@Autowired
	private PostfachMailController postfachMailController;
	@Autowired
	private CurrentUserService userService;
	@Autowired
	private ModelBuilderFactory<VorgangWithEingang> modelBuilderFactory;
	@Autowired
	private UserManagerUrlProvider userManagerUrlProvider;

	@Autowired
	private FeatureToggleProperties featureToggleProperties;
	@Autowired
	private VorgangProperties vorgangProperties;

	private static final Predicate<VorgangWithEingang> HAS_ATTACHMENTS = vorgangWithEingang -> vorgangWithEingang.getEingang()
			.getNumberOfAttachments() > 0;

	private static final Predicate<VorgangWithEingang> HAS_REPRESENTATIONS = vorgangWithEingang -> vorgangWithEingang.getEingang()
			.getNumberOfRepresentations() > 0;

	@Override
	public EntityModel<VorgangWithEingang> process(EntityModel<VorgangWithEingang> model) {
		var vorgang = model.getContent();

		if (Objects.isNull(vorgang)) {
			return model;
		}
		return modelBuilderFactory.fromModel(model)
				.addLink(linkTo(KommentarByVorgangController.class).slash(vorgang.getId()).slash("kommentars").withRel(REL_KOMMENTARE))
				.ifMatch(HAS_ATTACHMENTS)
				.addLink(vorgangWithEingang -> linkTo(methodOn(AttachmentController.class).getAllByEingang(vorgangWithEingang.getEingang().getId()))
						.withRel(REL_ATTACHMENTS))
				.ifMatch(HAS_REPRESENTATIONS)
				.addLink(vorgangWithEingang -> linkTo(
						methodOn(RepresentationController.class).getAllByEingang(vorgangWithEingang.getEingang().getId()))
								.withRel(REL_REPRESENTATIONS))
				.ifMatch(this::isPostfachConfigured)
				.addLink(linkTo(methodOn(PostfachMailController.class).getAll(vorgang.getId())).withRel(REL_POSTFACH_MAILS))
				.ifMatch(this::isEinheitlicherAnsprechpartner)
				.addLink(linkTo(methodOn(ForwardingController.class).findByVorgangId(vorgang.getId())).withRel(REL_VORGANG_FORWARDING))
				.addLink(linkTo(methodOn(HistorieController.class).getAll(vorgang.getId())).withRel(REL_HISTORIE))
				.ifMatch(() -> userManagerUrlProvider.isConfiguredForSearchUserProfile() && hasOrganisationsEinheitId(vorgang))
				.addLink(() -> buildSearchUserProfilesLink(vorgang))
				.ifMatch(() -> isCreateBescheidEnabled(vorgang))
				.addLink(linkTo(methodOn(CommandByRelationController.class).createCommand(vorgang.getId(), vorgang.getId(), vorgang.getVersion(),
						null)).withRel(REL_BESCHEID))
				.buildModel();
	}

	private boolean isPostfachConfigured(VorgangWithEingang vorgang) {
		return postfachMailController.isPostfachConfigured();
	}

	private boolean isEinheitlicherAnsprechpartner(VorgangWithEingang vorgang) {
		return userService.hasRole(UserRole.EINHEITLICHER_ANSPRECHPARTNER);
	}

	private Link buildSearchUserProfilesLink(VorgangWithEingang vorgang) {
		return Link.of(userManagerUrlProvider.getSearchUserProfilesTemplate()
				.queryParam(USER_PROFILE_SEARCH_DELETED_PARAM, false)
				.queryParam(USER_PROFILE_SEARCH_ORGANISATIONS_EINHEIT_ID_PARAM, getOrganisationsEinheitId(vorgang))
				.build(false).toUriString(), REL_SEARCH_USER);
	}

	private boolean hasOrganisationsEinheitId(VorgangWithEingang vorgang) {
		return Optional.of(vorgang.getEingang()).map(Eingang::getZustaendigeStelle).map(ZustaendigeStelle::getOrganisationseinheitenId).isPresent();
	}

	private String getOrganisationsEinheitId(VorgangWithEingang vorgang) {
		return vorgang.getEingang().getZustaendigeStelle().getOrganisationseinheitenId();
	}

	boolean isCreateBescheidEnabled(VorgangWithEingang vorgang) {
		return featureToggleProperties.isCreateBescheid() && hasVorgangCreateBescheidEnabled(vorgang);
	}

	boolean hasVorgangCreateBescheidEnabled(VorgangWithEingang vorgang) {
		return ofNullable(vorgang.getEingang())
				.map(Eingang::getHeader)
				.filter(HAS_FORM_ID_WITH_ENGINE_NAME)
				.map(this::isCreateBescheidEnabled)
				.orElse(false);
	}

	private boolean isCreateBescheidEnabled(EingangHeader eingangHeader) {
		return vorgangProperties.getBescheid().stream().filter(prop -> isFormIdAndFormEngineNameMatching(eingangHeader, prop)).count() > 0;
	}

	private boolean isFormIdAndFormEngineNameMatching(EingangHeader eingangHeader, VorgangProperty prop) {
		return prop.getFormId().equals(eingangHeader.getFormId()) && prop.getFormEngineName().equals(eingangHeader.getFormEngineName());
	}
}