package de.ozgcloud.alfa;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Collections;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.oauth2.jwt.Jwt;
import org.springframework.security.oauth2.jwt.JwtClaimNames;
import org.springframework.security.oauth2.server.resource.authentication.JwtAuthenticationToken;

import com.thedeanda.lorem.LoremIpsum;

class JwtAuthConverterTest {

	@Spy
	@InjectMocks
	private final JwtAuthConverter converter = new JwtAuthConverter();
	@Mock
	private OAuth2Properties oAuth2Properties;

	@DisplayName("Convert")
	@Nested
	class TestConvert {

		@Mock
		private Jwt jwt;

		@BeforeEach
		void mock() {
			doReturn(StringUtils.EMPTY).when(converter).getPrincipleClaimName(any());
			doReturn(Collections.emptySet()).when(converter).getAuthorities(any());
		}

		@Test
		void shouldGetAuthrorities() {
			converter.convert(jwt);

			verify(converter).getAuthorities(jwt);
		}

		@Test
		void shouldGetPrincipleClaimName() {
			converter.convert(jwt);

			verify(converter).getPrincipleClaimName(jwt);
		}

		@Test
		void shouldReturnJwtAuthenticationToken() {
			var token = converter.convert(jwt);

			assertThat(token).isInstanceOf(JwtAuthenticationToken.class);
		}
	}

	@DisplayName("Get principle claim name")
	@Nested
	class TestGetPrincipleClaimName {

		private final String subClaimValue = LoremIpsum.getInstance().getWords(1);
		private final String principleClaimKey = LoremIpsum.getInstance().getWords(1);
		private final String principleClaimValue = LoremIpsum.getInstance().getWords(1);
		private final Map<String, Object> claimMap = Map.of(JwtClaimNames.SUB, subClaimValue, principleClaimKey, principleClaimValue);
		private final Jwt jwt = JwtTestFactory.create(claimMap);

		@DisplayName("if principle attribute is set")
		@Nested
		class TestIfPrincipleAttributeIsSet {

			@BeforeEach
			void mock() {
				when(oAuth2Properties.getPrincipleAttribute()).thenReturn(null);
			}

			@Test
			void shouleReturnPrincipleAttributeClaimValue() {
				var principleClaimName = converter.getPrincipleClaimName(jwt);

				assertThat(principleClaimName).isEqualTo(subClaimValue);
			}
		}

		@DisplayName("if principle attribute is not set")
		@Nested
		class TestIfPrincipleAttributeNotSet {

			@BeforeEach
			void mock() {
				when(oAuth2Properties.getPrincipleAttribute()).thenReturn(principleClaimKey);
			}

			@Test
			void shouldReturnSubClaimValue() {
				var principleClaimName = converter.getPrincipleClaimName(jwt);

				assertThat(principleClaimName).isEqualTo(principleClaimValue);
			}
		}
	}

	@DisplayName("Get authorities")
	@Nested
	class TestGetAuthorities {

		private final static String SCOPE_CLAIM_KEY = "scope";
		private final static String SCOPE_CLAIM_VALUE = "dummyClaimValue";
		private final Jwt jwt = JwtTestFactory.create(Map.of(SCOPE_CLAIM_KEY, SCOPE_CLAIM_VALUE));

		@BeforeEach
		void mock() {
			doReturn(Collections.emptyList()).when(converter).extractResourceRoles(any());
		}

		@Test
		void shouldConvertJwt() {
			var authorities = converter.getAuthorities(jwt);

			assertThat(authorities).extracting(GrantedAuthority::toString)
					.contains(String.format("%s_%s", SCOPE_CLAIM_KEY.toUpperCase(), SCOPE_CLAIM_VALUE));
		}

		@Test
		void shouldExtractRoles() {
			converter.getAuthorities(jwt);

			verify(converter).extractResourceRoles(jwt);
		}
	}

	@DisplayName("Extract resource roles")
	@Nested
	class TestExtractResourceRoles {

		private final Map<String, Object> claimsMap = Map.of(JwtAuthConverter.RESOURCE_ACCESS_KEY, Map.of("dummyKey", "dummyValue"));
		private final Jwt jwt = JwtTestFactory.create(claimsMap);

		@Test
		void shouldReturnEmptySetOnMissingResourceAccess() {
			var claimsMap = Map.<String, Object>of(JwtAuthConverter.RESOURCE_ACCESS_KEY, Collections.emptyMap());
			var jwt = JwtTestFactory.create(claimsMap);

			var resourceRoles = converter.extractResourceRoles(jwt);

			assertThat(resourceRoles).isEmpty();
		}

		@DisplayName("on existing resource access")
		@Nested
		class TestOnExistingResourceAccess {

			@Test
			void shouldCallProperties() {
				converter.extractResourceRoles(jwt);

				verify(oAuth2Properties).getResource();
			}

			@Test
			void shouldReturnEmptySetOnMissingResourceId() {
				when(oAuth2Properties.getResource()).thenReturn(null);

				var resourceRoles = converter.extractResourceRoles(jwt);

				assertThat(resourceRoles).isEmpty();
			}

			@DisplayName("and existing resourceId")
			@Nested
			class TestAndExistingResourceId {

				private final static String RESOURCE_ID = "dummyResourceId";
				private final Map<String, Object> authoritiesMap = Map.of(JwtAuthConverter.RESOURCE_ACCESS_KEY,
						Map.of(RESOURCE_ID, Collections.emptyMap()));
				private final Jwt jwt = JwtTestFactory.create(authoritiesMap);

				@BeforeEach
				void mock() {
					doReturn(Collections.emptySet()).when(converter).extractRoles(any());
					when(oAuth2Properties.getResource()).thenReturn(RESOURCE_ID);
				}

				@Test
				void shouldExtractRoles() {
					converter.extractResourceRoles(jwt);

					verify(converter).extractRoles(any());
				}
			}

		}
	}

	@DisplayName("Extract roles")
	@Nested
	class TestExtractRoles {

		@Test
		void shouldReturnMappedRoleIfRolesExist() {
			var authorities = converter.extractRoles(Map.of(JwtAuthConverter.ROLES_KEY, List.of("dummy")));

			assertThat(authorities).extracting(SimpleGrantedAuthority::toString).containsExactly("ROLE_dummy");
		}

		@Test
		void shouldReturnEmptySetIfNoRoleExists() {
			var authorities = converter.extractRoles(Map.of(JwtAuthConverter.ROLES_KEY, Collections.emptyList()));

			assertThat(authorities).isEmpty();
		}

		@Test
		void shouldReturnEmptySetIfRolesEntryNotExists() {
			var authorities = converter.extractRoles(Collections.emptyMap());

			assertThat(authorities).isEmpty();
		}
	}
}
