/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common;

import static org.assertj.core.api.Assertions.*;

import java.lang.reflect.Field;
import java.util.List;
import java.util.UUID;

import org.apache.commons.lang3.reflect.FieldUtils;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.Link;
import org.springframework.web.bind.annotation.RequestMapping;

import lombok.Builder;

class ModelBuilderTest {

	private static final String USER_MANAGER_PROFILE_TEMPLATE = "/api/profile/%s";

	@DisplayName("Build model")
	@Nested
	class TestBuildModel {

		private final TestEntity entity = TestEntityTestFactory.create();

		@Test
		void shouldHaveAddLinkByLinkedResource() {
			var model = buildModel();

			assertThat(model.getLink(TestController.FILE_REL).get().getHref()).isEqualTo(TestController.PATH + "/" + TestEntityTestFactory.FILE);
		}

		@Test
		void shouldHaveAddLinkByLinkedUserProfileResource() {
			var model = buildModel();

			assertThat(model.getLink(TestController.USER_REL).get().getHref())
					.isEqualTo(String.format(USER_MANAGER_PROFILE_TEMPLATE, TestEntityTestFactory.USER));
		}

		private EntityModel<TestEntity> buildModel() {
			return ModelBuilder.fromEntity(entity).withUserManagerUrl(USER_MANAGER_PROFILE_TEMPLATE).buildModel();
		}

		@DisplayName("if user manager is configured")
		@Nested
		class TestIfUserManagerIsConfigured {

			@Test
			void shouldAddLinkByLinkedUserProfileResourceAnnotation() {
				var builder = ModelBuilder.fromEntity(TestEntityTestFactory.create()).withUserManagerUrl(USER_MANAGER_PROFILE_TEMPLATE);

				var model = builder.buildModel();

				var expectedUserLink = Link.of(String.format(USER_MANAGER_PROFILE_TEMPLATE, TestEntityTestFactory.USER))
						.withRel(TestController.USER_REL);
				assertThat(model.getLinks()).contains(expectedUserLink);
			}
		}

		@DisplayName("if user manager is NOT configured")
		@Nested
		class TestIfUserManagerIsNotConfigured {

			@Test
			void shouldNotAddLinkByLinkedUserProfileResourceAnnotation() {
				var builder = ModelBuilder.fromEntity(TestEntityTestFactory.create());

				var model = builder.buildModel();

				assertThat(model.getLinks().hasLink(TestController.USER_REL)).isFalse();
			}
		}
	}

	@DisplayName("Add link by linkedUserProfileResource annotation")
	@Nested
	class TestHandleLinkedUserProfileResourceField {

		private List<Field> annotationFields = FieldUtils.getFieldsListWithAnnotation(TestEntity.class, LinkedUserProfileResource.class);

		@Test
		void shouldAddLinkForUserProfile() {
			var entityModel = EntityModel.of(TestEntityTestFactory.create());
			var builder = ModelBuilder.fromEntity(TestEntityTestFactory.create()).withUserManagerUrl(USER_MANAGER_PROFILE_TEMPLATE);

			builder.handleLinkedUserProfileResourceField(entityModel, annotationFields.get(0));

			var expectedUserLink = Link.of(String.format(USER_MANAGER_PROFILE_TEMPLATE, TestEntityTestFactory.USER)).withRel(TestController.USER_REL);
			assertThat(entityModel.getLinks()).contains(expectedUserLink);
		}
	}
}

@Builder
class TestEntity {

	@LinkedResource(controllerClass = TestController.class)
	private String file;

	@LinkedUserProfileResource
	private String user;
}

@RequestMapping(TestController.PATH)
class TestController {

	static final String PATH = "/api/test";

	static final String USER_REL = "user";
	static final String FILE_REL = "file";

}

class TestEntityTestFactory {

	static final String USER = UUID.randomUUID().toString();
	static final String FILE = UUID.randomUUID().toString();

	public static TestEntity create() {
		return TestEntity.builder()
				.file(FILE)
				.user(USER)
				.build();
	}
}
