/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.binaryfile;

import static org.assertj.core.api.Assertions.*;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.io.OutputStream;
import java.util.List;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.ValueSource;
import org.mockito.ArgumentMatchers;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import com.google.protobuf.ByteString;

import de.itvsh.kop.common.errorhandling.TechnicalException;
import de.itvsh.ozg.pluto.grpc.binaryFile.BinaryFileServiceGrpc.BinaryFileServiceBlockingStub;
import de.itvsh.ozg.pluto.grpc.binaryFile.BinaryFileServiceGrpc.BinaryFileServiceStub;
import de.itvsh.ozg.pluto.grpc.binaryFile.GrpcFindFilesResponse;
import de.itvsh.ozg.pluto.grpc.binaryFile.GrpcGetBinaryFileDataRequest;
import de.itvsh.ozg.pluto.grpc.binaryFile.GrpcUploadBinaryFileMetaData;
import de.itvsh.ozg.pluto.grpc.binaryFile.GrpcUploadBinaryFileRequest;
import de.itvsh.ozg.pluto.grpc.command.GrpcCallContext;
import de.itvsh.ozg.pluto.grpc.file.GrpcOzgFile;
import de.ozgcloud.alfa.common.GrpcCallContextTestFactory;
import de.ozgcloud.alfa.common.callcontext.ContextService;
import de.ozgcloud.alfa.common.file.GrpcOzgFileTestFactory;
import de.ozgcloud.alfa.common.file.OzgFile;
import de.ozgcloud.alfa.common.file.OzgFileMapper;
import de.ozgcloud.alfa.common.file.OzgFileTestFactory;
import io.grpc.stub.CallStreamObserver;

class BinaryFileRemoteServiceTest {

	@Spy
	@InjectMocks
	private BinaryFileRemoteService remoteService;
	@Mock
	private BinaryFileServiceBlockingStub serviceStub;
	@Mock
	private BinaryFileServiceStub serviceAsyncStub;
	@Mock
	private ContextService contextService;
	@Mock
	private OzgFileMapper ozgFileMapper;
	@Mock
	private FileIdMapper fileIdMapper;

	private GrpcCallContext callContext = GrpcCallContextTestFactory.create();

	@BeforeEach
	void mockContextService() {
		lenient().when(contextService.createCallContext()).thenReturn(callContext);
	}

	@Nested
	class TestUploadBinaryFile {

		@Mock
		private CallStreamObserver<GrpcUploadBinaryFileRequest> requestObserver;
		@Mock
		private BinaryFileUploadStreamObserver responseObserver;
		private UploadBinaryFileRequest uploadBinaryFile = UploadBinaryFileTestFactory.create();

		@BeforeEach
		void initMocks() {
			when(serviceAsyncStub.uploadBinaryFileAsStream(any())).thenReturn(requestObserver);
		}

		@Test
		void shouldCreateStreamObserver() {
			callService();

			verify(remoteService).createUploadBinaryFileObserver(ArgumentMatchers.<CompletableFuture<FileId>>any(), any());
			verify(remoteService).buildMetaDataRequest(uploadBinaryFile);
		}

		@Test
		void shouldCallUploadBinaryFileOnServiceAsyncStub() {
			doReturn(responseObserver).when(remoteService).createUploadBinaryFileObserver(any(), any());

			callService();

			verify(serviceAsyncStub).uploadBinaryFileAsStream(responseObserver);
		}

		private void callService() {
			remoteService.uploadFile(uploadBinaryFile);
		}
	}

	@Nested
	class TestBuildMetaDataRequest {

		@Test
		void shouldCallContextService() {
			callService();

			verify(contextService).createCallContext();
		}

		@Test
		void shouldContainsVorgangId() {
			var metaData = callService();

			assertThat(metaData.getVorgangId()).isEqualTo(UploadBinaryFileTestFactory.VORGANG_ID);
		}

		@Test
		void shouldContainsFieldName() {
			var metaData = callService();

			assertThat(metaData.getField()).isEqualTo(UploadBinaryFileTestFactory.FIELD);
		}

		@Test
		void shouldContainsContentType() {
			var metaData = callService();

			assertThat(metaData.getContentType()).isEqualTo(OzgFileTestFactory.CONTENT_TYPE);

		}

		@Test
		void shouldContainsSize() {
			var metaData = callService();

			assertThat(metaData.getSize()).isEqualTo(OzgFileTestFactory.SIZE);
		}

		@Test
		void shouldContainsFileName() {
			var metaData = callService();

			assertThat(metaData.getFileName()).isEqualTo(OzgFileTestFactory.NAME);
		}

		private GrpcUploadBinaryFileMetaData callService() {
			return remoteService.buildMetaDataRequest(UploadBinaryFileTestFactory.create()).getMetadata();
		}
	}

	@Nested
	class TestBuildChunkRequest {

		@Test
		void shouldContainContent() {
			var chunkRequest = remoteService.buildChunkRequest(BinaryFileTestFactory.DATA);

			assertThat(chunkRequest.getFileContent()).isEqualTo(ByteString.copyFrom(BinaryFileTestFactory.DATA));
		}
	}

	@Nested
	class TestGetFile {

		private final GrpcFindFilesResponse response = BinaryFileTestFactory.createFindBinaryFilesResponse();

		@BeforeEach
		void mock() {
			when(serviceStub.findBinaryFilesMetaData(any())).thenReturn(response);
		}

		@Test
		void shouldCallServiceStub() {
			doServiceCall();

			verify(serviceStub).findBinaryFilesMetaData(any());
		}

		@Test
		void shouldCallContextService() {
			doServiceCall();

			verify(contextService).createCallContext();
		}

		@Test
		void shouldCallMapper() {
			doServiceCall();

			verify(ozgFileMapper).toFile(GrpcOzgFileTestFactory.create());
		}

		private OzgFile doServiceCall() {
			return remoteService.getFile(BinaryFileTestFactory.FILE_ID);
		}
	}

	@Nested
	class TestGetFileData {

		@Mock
		private OutputStream out;
		@Mock
		private BinaryFileDownloadStreamObserver responseObserver;
		private String fileId = OzgFileTestFactory.ID.toString();

		@BeforeEach
		void mockStreamObserver() {
			doReturn(responseObserver).when(remoteService).createDownloadBinaryFileObserver(any(), any());
			when(fileIdMapper.toString(any())).thenReturn(fileId);
			doNothing().when(remoteService).waitUntilFutureToComplete(any());
		}

		@Test
		void shouldCreateDownloadBinaryFileObserver() {
			callService();

			verify(remoteService).createDownloadBinaryFileObserver(ArgumentMatchers.<CompletableFuture<Boolean>>any(), eq(out));
		}

		@Test
		void shouldCallContextService() {
			callService();

			verify(contextService).createCallContext();
		}

		@Test
		void shouldCallFileIdMapper() {
			callService();

			verify(fileIdMapper).toString(OzgFileTestFactory.ID);
		}

		@Test
		void shouldCallGetFileDataOnAsyncServiceStub() {
			callService();

			verify(serviceAsyncStub).getBinaryFileContent(any(GrpcGetBinaryFileDataRequest.class), eq(responseObserver));
		}

		@Test
		void shouldWaitUntilFutureToComplete() {
			callService();

			verify(remoteService).waitUntilFutureToComplete(any());
		}

		private void callService() {
			remoteService.writeFileContent(OzgFileTestFactory.ID, out);
		}
	}

	@Nested
	class TestWaitUntilFutureToComplete {

		@Mock
		private CompletableFuture<Boolean> streamFuture;

		@Test
		void shouldNotThrowException() {
			assertDoesNotThrow(() -> remoteService.waitUntilFutureToComplete(streamFuture));
		}

		@ParameterizedTest
		@ValueSource(classes = { InterruptedException.class, ExecutionException.class, TimeoutException.class })
		void shouldRethrowAsTechnicalException(Class<Exception> exception)
				throws InterruptedException, ExecutionException, TimeoutException {
			doThrow(exception).when(streamFuture).get(anyLong(), any(TimeUnit.class));

			assertThrows(TechnicalException.class, () -> remoteService.waitUntilFutureToComplete(streamFuture));
		}
	}

	@Nested
	class TestGetFiles {

		private final GrpcFindFilesResponse response = BinaryFileTestFactory.createFindBinaryFilesResponse();

		@BeforeEach
		void mock() {
			when(serviceStub.findBinaryFilesMetaData(any())).thenReturn(response);
			when(fileIdMapper.toString(any())).thenReturn(OzgFileTestFactory.ID.toString());
		}

		@Test
		void shouldCallContextService() {
			doServiceCall();

			verify(contextService).createCallContext();
		}

		@Test
		void shouldCallFileIdMapper() {
			doServiceCall();

			verify(fileIdMapper).toString(any());
		}

		@Test
		void shouldCallOzgFileMapper() {
			var result = doServiceCall();

			result = result.collect(Collectors.toList()).stream();

			verify(ozgFileMapper).toFile(any(GrpcOzgFile.class));
		}

		private Stream<OzgFile> doServiceCall() {
			return remoteService.getFiles(List.of(OzgFileTestFactory.ID));
		}
	}
}