/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.kommentar;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.itvsh.kop.pluto.common.grpc.GrpcObjectMapper;
import de.ozgcloud.alfa.common.attacheditem.VorgangAttachedItemTestFactory;
import de.ozgcloud.alfa.common.binaryfile.BinaryFileTestFactory;
import de.ozgcloud.alfa.vorgang.GrpcVorgangAttachedItemTestFactory;

class KommentarMapperTest {

	@Spy
	@InjectMocks
	private KommentarMapper mapper = Mappers.getMapper(KommentarMapper.class);
	@Mock
	private GrpcObjectMapper grpcObjectMapper;

	@DisplayName("Map from item")
	@Nested
	class TestFromItem {

		@BeforeEach
		void mockMapper() {
			when(grpcObjectMapper.mapFromGrpc(any())).thenReturn(KommentarTestFactory.createAsMap());
		}

		@Test
		void shouldCallGrpcObjectMapper() {
			mapper.fromItem(GrpcVorgangAttachedItemTestFactory.create());

			verify(grpcObjectMapper).mapFromGrpc(GrpcVorgangAttachedItemTestFactory.ITEM);
		}

		@Test
		void shouldCallMapperFromItemMap() {
			mapper.fromItem(GrpcVorgangAttachedItemTestFactory.create());

			verify(mapper).fromItemMap(KommentarTestFactory.createAsMap(), VorgangAttachedItemTestFactory.VERSION);
		}
	}

	@DisplayName("Map from item map")
	@Nested
	class TestFromItemMap {

		@Test
		void shouldMapId() {
			var kommentar = map();

			assertThat(kommentar.getId()).isEqualTo(KommentarTestFactory.ID);
		}

		@Test
		void shouldMapVersion() {
			var kommentar = map();

			assertThat(kommentar.getVersion()).isEqualTo(KommentarTestFactory.VERSION);
		}

		@Test
		void shouldMapCreatedBy() {
			var kommentar = map();

			assertThat(kommentar.getCreatedBy()).isEqualTo(KommentarTestFactory.CREATED_BY);
		}

		@Test
		void shouldMapCreatedAt() {
			var kommentar = map();

			assertThat(kommentar.getCreatedAt()).isEqualTo(KommentarTestFactory.CREATED_AT);
		}

		@Test
		void shouldMapText() {
			var kommentar = map();

			assertThat(kommentar.getText()).isEqualTo(KommentarTestFactory.TEXT);
		}

		@Test
		void shouldMapNullAttachmentsToEmptyList() {
			var kommentar = mapper.fromItemMap(createAsMapWithAttachmentIds(null), KommentarTestFactory.VERSION);

			assertThat(kommentar.getAttachments()).isEmpty();
		}

		@Test
		void shouldMapEmptyAttachmentsToEmptyList() {
			var kommentar = mapper.fromItemMap(createAsMapWithAttachmentIds(Collections.emptyList()), KommentarTestFactory.VERSION);

			assertThat(kommentar.getAttachments()).isEmpty();
		}

		@Test
		void shouldMapAttachmentsToList() {
			var kommentar = mapper.fromItemMap(
					KommentarTestFactory.createAsMap(),
					KommentarTestFactory.VERSION);

			assertThat(kommentar.getAttachments()).containsOnly(BinaryFileTestFactory.FILE_ID);
		}

	}

	private Kommentar map() {
		return mapper.fromItemMap(KommentarTestFactory.createAsMap(), KommentarTestFactory.VERSION);
	}

	private static Map<String, Object> createAsMapWithAttachmentIds(Collection<String> attachmentIds) {
		var map = new HashMap<>(KommentarTestFactory.createAsMap());
		map.replace(KommentarMapper.ATTACHMENTS, attachmentIds);
		return Collections.unmodifiableMap(map);
	}
}
