/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.kommentar;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.time.ZonedDateTime;
import java.time.temporal.ChronoUnit;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.alfa.common.attacheditem.VorgangAttachedItemService;
import de.ozgcloud.alfa.common.binaryfile.BinaryFileTestFactory;
import de.ozgcloud.alfa.common.command.Command;
import de.ozgcloud.alfa.common.command.CommandTestFactory;
import de.ozgcloud.alfa.common.user.CurrentUserService;
import de.ozgcloud.alfa.common.user.UserProfileTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;

class KommentarServiceTest {

	@Spy
	@InjectMocks
	private KommentarService service;
	@Mock
	private KommentarRemoteService remoteService;
	@Mock
	private VorgangAttachedItemService vorgangAttachedItemService;
	@Mock
	private CurrentUserService currentUserService;

	@DisplayName("Create kommentar")
	@Nested
	class TestCreateKommentar {

		@DisplayName("should")
		@Nested
		class TestCalls {

			@BeforeEach
			void mockServices() {
				doReturn(KommentarTestFactory.create()).when(service).addCreated(any());
				when(vorgangAttachedItemService.createNewKommentar(any(), any())).thenReturn(CommandTestFactory.create());
			}

			@DisplayName("add created")
			@Test
			void shouldAddCreated() {
				callCreateKommentar();

				verify(service).addCreated(any(Kommentar.class));
			}

			@DisplayName("call vorgangattacheditem service")
			@Test
			void shouldCallVorgangAttachedItemService() {
				callCreateKommentar();

				verify(vorgangAttachedItemService).createNewKommentar(any(Kommentar.class), eq(VorgangHeaderTestFactory.ID));
			}
		}

		@DisplayName("Add created")
		@Nested
		class TestAddCreated {

			@BeforeEach
			void mockServices() {
				when(currentUserService.getUserId()).thenReturn(UserProfileTestFactory.ID);
			}

			@Test
			void shouldSetCreatedAt() throws Exception {
				var kommentar = callAddCreated();

				assertThat(kommentar.getCreatedAt()).isNotNull().isCloseTo(ZonedDateTime.now(), within(2, ChronoUnit.SECONDS));
			}

			@Test
			void shouldSetCreatedBy() throws Exception {
				var kommentar = callAddCreated();

				assertThat(kommentar.getCreatedBy()).isEqualTo(UserProfileTestFactory.ID.toString());
			}

			private Kommentar callAddCreated() {
				return service.addCreated(KommentarTestFactory.createBuilder().createdAt(null).createdBy(null).build());
			}
		}

		private Command callCreateKommentar() {
			return service.createKommentar(KommentarTestFactory.create(), VorgangHeaderTestFactory.ID);
		}
	}

	@DisplayName("Edit kommentar")
	@Nested
	class TestEditKommentar {

		@Captor
		private ArgumentCaptor<Kommentar> kommentarCaptor;

		@BeforeEach
		void mockKommentarService() {
			when(remoteService.getById(anyString())).thenReturn(KommentarTestFactory.createBuilder().text("text needs to be override").build());
		}

		@Test
		void shouldCallGetById() {
			callEditKommentar();

			verify(service).getById(KommentarTestFactory.ID);
		}

		@Test
		void shouldReplaceText() {
			callEditKommentar();

			verify(vorgangAttachedItemService).editKommentar(kommentarCaptor.capture(), eq(KommentarTestFactory.ID),
					eq(KommentarTestFactory.VERSION));
			assertThat(kommentarCaptor.getValue().getText()).isEqualTo(KommentarTestFactory.TEXT);
		}

		@Test
		void shouldReplaceAttachments() {
			service.editKommentar(KommentarTestFactory.create(), KommentarTestFactory.ID, KommentarTestFactory.VERSION);

			verify(vorgangAttachedItemService).editKommentar(
					kommentarCaptor.capture(),
					eq(KommentarTestFactory.ID),
					eq(KommentarTestFactory.VERSION));
			assertThat(kommentarCaptor.getValue().getAttachments()).containsExactly(BinaryFileTestFactory.FILE_ID);
		}

		@Test
		void shouldCallVorgangAttachedItemService() {
			callEditKommentar();

			verify(vorgangAttachedItemService).editKommentar(any(Kommentar.class), eq(KommentarTestFactory.ID), eq(KommentarTestFactory.VERSION));
		}

		private Command callEditKommentar() {
			return service.editKommentar(KommentarTestFactory.create(), KommentarTestFactory.ID, KommentarTestFactory.VERSION);
		}
	}

	@Nested
	class TestGetById {

		@Test
		void shouldCallRemoteService() {
			service.getById(KommentarTestFactory.ID);

			verify(remoteService).getById(KommentarTestFactory.ID);
		}
	}

	@Nested
	class TestFindByVorgangId {

		@Test
		void shouldCallRemoteService() {
			service.findByVorgangId(VorgangHeaderTestFactory.ID);

			verify(remoteService).findByVorgangId(VorgangHeaderTestFactory.ID);
		}
	}
}
