package de.ozgcloud.alfa.statistic;

import static de.ozgcloud.alfa.statistic.StatisticRemoteService.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Collections;
import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.EnumSource;
import org.junit.jupiter.params.provider.EnumSource.Mode;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.itvsh.ozg.pluto.statistic.GrpcVorgangCountResponse;
import de.itvsh.ozg.pluto.statistic.GrpcVorgangStatisticQuery;
import de.itvsh.ozg.pluto.statistic.GrpcVorgangStatisticQuery.GroupMethod;
import de.itvsh.ozg.pluto.statistic.GrpcVorgangStatisticQuery.Operator;
import de.itvsh.ozg.pluto.statistic.GrpcVorgangStatisticResponse;
import de.itvsh.ozg.pluto.statistic.GrpcVorgangStatisticResult;
import de.itvsh.ozg.pluto.statistic.StatisticServiceGrpc.StatisticServiceBlockingStub;
import de.ozgcloud.alfa.vorgang.Vorgang.VorgangStatus;

class StatisticRemoteServiceTest {

	@Spy
	@InjectMocks
	private StatisticRemoteService service;
	@Mock
	private StatisticServiceBlockingStub serviceStub;
	@Mock
	private StatisticMapper mapper;

	private static final List<VorgangStatus> STATUSES_TO_COUNT = List.of(VorgangStatus.NEU, VorgangStatus.ANGENOMMEN,
			VorgangStatus.IN_BEARBEITUNG, VorgangStatus.BESCHIEDEN,
			VorgangStatus.ABGESCHLOSSEN, VorgangStatus.VERWORFEN, VorgangStatus.ZU_LOESCHEN);

	@Nested
	class TestBuildCountByVorgangStatusQuery {

		@ParameterizedTest
		@EnumSource(mode = Mode.EXCLUDE, names = "WEITERGELEITET")
		void shouldBuildQuery(VorgangStatus vorgangStatus) {
			var query = service.buildCountByVorgangStatusQuery(vorgangStatus);

			assertThat(query)
					.extracting(
							GrpcVorgangStatisticQuery::getResultName,
							GrpcVorgangStatisticQuery::getPath,
							GrpcVorgangStatisticQuery::getOperator,
							GrpcVorgangStatisticQuery::getOperandStringValue,
							GrpcVorgangStatisticQuery::getGroupMethod)
					.containsExactly(
							String.format(COUNT_VORGANG_STATUS_RESULT_NAME_FORMAT, vorgangStatus.name()),
							STATUS_PATH,
							Operator.EQUAL,
							vorgangStatus.name(),
							GroupMethod.COUNT);
		}
	}

	@Nested
	class TestBuildCountByStatusQueries {

		@ParameterizedTest
		@EnumSource(mode = Mode.EXCLUDE, names = "WEITERGELEITET")
		void shouldCallBuildCountByVorgangStatusQuery(VorgangStatus vorgangStatus) {
			service.buildCountByStatusQueries(STATUSES_TO_COUNT);

			verify(service).buildCountByVorgangStatusQuery(vorgangStatus);
		}
	}

	@Nested
	class TestBuildExistsWiedervorlageOverdueQuery {

		@Test
		void shouldBuildQuery() {
			var query = service.buildExistsWiedervorlageOverdueQuery();

			assertThat(query)
					.extracting(
							GrpcVorgangStatisticQuery::getResultName,
							GrpcVorgangStatisticQuery::getPath,
							GrpcVorgangStatisticQuery::getOperator,
							GrpcVorgangStatisticQuery::getGroupMethod,
							GrpcVorgangStatisticQuery::getOperandStringValue)
					.contains(
							EXISTS_WIEDERVORLAGE_OVERDUE_RESULT_NAME,
							WIEDERVORLAGE_NEXT_FRIST_PATH,
							Operator.LESS_THEN_OR_EQUAL_TO,
							GroupMethod.EXISTS,
							OPERAND_TODAY_DATE);
		}
	}

	@Nested
	class TestGetVorgaengeStatistics {

		private GrpcVorgangStatisticResponse response;

		@BeforeEach
		void beforeEach() {
			response = GrpcVorgangStatisticResponse.newBuilder().build();
			when(serviceStub.getVorgangStatistic(any())).thenReturn(response);
		}

		@Test
		void shouldCallServiceStub() {
			service.getVorgaengeStatistics(Collections.emptyList());

			verify(serviceStub).getVorgangStatistic(any());
		}

		@Test
		void shouldCallBuildCountByStatusQueries() {
			service.getVorgaengeStatistics(STATUSES_TO_COUNT);

			verify(service).buildCountByStatusQueries(STATUSES_TO_COUNT);
		}

		@Test
		void shouldCallBuildExistsWiedervorlageOverdueQuery() {
			service.getVorgaengeStatistics(STATUSES_TO_COUNT);

			verify(service).buildExistsWiedervorlageOverdueQuery();
		}

		@Test
		void shouldCallBuildStatisticResult() {
			service.getVorgaengeStatistics(STATUSES_TO_COUNT);

			verify(service).buildStatisticResult(response);
		}
	}

	@Nested
	class TestBuildStatisticResult {

		private final String RESULT_NAME = "countByStatus";
		private final int RESULT_INT = 42;
		private GrpcVorgangStatisticResponse grpcResponse;
		private GrpcVorgangStatisticResult grpcResult;

		@BeforeEach
		void beforeEach() {
			grpcResult = GrpcVorgangStatisticResult.newBuilder()
					.setName(RESULT_NAME)
					.setResultIntValue(RESULT_INT)
					.build();
			grpcResponse = GrpcVorgangStatisticResponse.newBuilder()
					.addResult(grpcResult)
					.build();
			when(mapper.toResult(grpcResult)).thenReturn(StatisticResult.builder().name(RESULT_NAME).intValue(RESULT_INT).build());
		}

		@Test
		void shouldCallMapper() {
			service.buildStatisticResult(grpcResponse);

			verify(mapper).toResult(grpcResult);
		}

		@Test
		void shouldBuildResult() {
			var result = service.buildStatisticResult(grpcResponse);

			assertThat(result)
					.hasSize(1)
					.hasEntrySatisfying(RESULT_NAME, statisticResult -> assertThat(statisticResult.getIntValue()).isEqualTo(RESULT_INT));
		}
	}

	// TODO sebo: später löschen
	@DisplayName("Get statistic")
	@Nested
	class TestGetStatistic {

		private String countByPath = "DummyCountPath";
		private List<String> countByPathList = List.of(countByPath);
		private final GrpcVorgangCountResponse response = GrpcVorgangCountTestFactory.createResponse();

		@DisplayName("prozess flow")
		@Nested
		class TestProzessFlow {

			@BeforeEach
			void mockServiceStub() {
				when(serviceStub.countVorgang(any())).thenReturn(response);
			}

			@Test
			void shouldCallStub() {
				service.countVorgang(countByPathList);

				verify(serviceStub).countVorgang(any());
			}

			@Test
			void shouldBuildRequest() {
				service.countVorgang(countByPathList);

				verify(service).buildVorgangCountRequest(countByPathList);
			}

			@Test
			void shouldMapCountVorgangResponse() {
				service.countVorgang(countByPathList);

				verify(mapper).fromCountVorgangResponse(response);
			}
		}

		@DisplayName("build vorgang count request")
		@Nested
		class TestBuildVorgangCountRequest {

			@Test
			void shouldHaveCountByPath() {
				var request = service.buildVorgangCountRequest(countByPathList);

				assertThat(request.getCountByPathList()).hasSize(1).containsExactly(countByPath);
			}
		}
	}

}