/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.vorgang;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Collections;
import java.util.List;
import java.util.Map;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.itvsh.kop.pluto.common.grpc.GrpcFormDataMapper;
import de.itvsh.ozg.pluto.vorgang.GrpcFormData;
import de.itvsh.ozg.pluto.vorgang.GrpcFormField;
import de.itvsh.ozg.pluto.vorgang.GrpcSubForm;

class GrpcFormDataMapperTest {

	@InjectMocks
	private GrpcFormDataMapper MAPPER_INSTANCE = Mappers.getMapper(GrpcFormDataMapper.class);

	@Mock
	private GrpcFormDataMapper grpcFormDataMapper;

	@BeforeEach
	void mockMapperReturnValues() {
		lenient().when(grpcFormDataMapper.mapToFormData(any()))
				.thenReturn(GrpcFormData.newBuilder().addField(GrpcFormFieldTestFactory.create()).build());
	}

	@Nested
	class TestSimpleValueMapping {

		@Test
		void shouldHaveField() {

			GrpcFormData formData = MAPPER_INSTANCE.mapToFormData(Map.of("key", "value"));

			assertThat(formData.getFieldList()).hasSize(1);
		}

		@Test
		void shouldNotHaveSubForms() {

			GrpcFormData formData = MAPPER_INSTANCE.mapToFormData(Map.of("key", "value"));

			assertThat(formData.getFormList()).isEmpty();
		}

		@Test
		void shouldHaveNameField() {

			GrpcFormData formData = MAPPER_INSTANCE.mapToFormData(Map.of("key", "value"));

			assertThat(formData.getField(0).getName()).isEqualTo("key");
			assertThat(formData.getField(0).getValue()).isEqualTo("value");
		}
	}

	@Nested
	class TestSubFormMapping {

		@Test
		void shouldHaveSubForm() {

			GrpcFormData formData = MAPPER_INSTANCE.mapToFormData(Map.of("key", Map.of("subKey", "value")));

			assertThat(formData.getFormList()).hasSize(1);
		}

		@Test
		void shouldNotHaveFields() {

			GrpcFormData formData = MAPPER_INSTANCE.mapToFormData(Map.of("key", Map.of("subKey", "value")));

			assertThat(formData.getFieldList()).isEmpty();
		}

		@Test
		void shouldHaveSubFormField() {

			GrpcFormData formData = MAPPER_INSTANCE.mapToFormData(Map.of("key", Map.of("subKey", "value")));

			assertThat(formData.getForm(0).getFieldList()).hasSize(1);
			assertThat(formData.getForm(0).getTitle()).isEqualTo("key");
			assertThat(formData.getForm(0).getField(0).getName()).isEqualTo("subKey");
			assertThat(formData.getForm(0).getField(0).getValue()).isEqualTo("value");
		}
	}

	@Nested
	class TestMapStringListsToFields {

		@Test
		void emptyMapShouldNotBeMapped() {

			List<GrpcFormField> fields = MAPPER_INSTANCE.mapStringListsToFields(Collections.emptyMap());

			assertThat(fields).isEmpty();
		}

		@Test
		void simpleValueShouldNotBeMapped() {

			List<GrpcFormField> fields = MAPPER_INSTANCE.mapStringListsToFields(Map.of("a", "b"));

			assertThat(fields).isEmpty();
		}

		@Test
		void listObjectValuesShouldBeMapped() {

			List<GrpcFormField> fields = MAPPER_INSTANCE.mapStringListsToFields(Map.of("key", List.of("value1", "value2")));

			assertThat(fields).hasSize(2);
			assertThat(fields.get(0).getName()).isEqualTo("key");
			assertThat(fields.get(0).getValue()).isEqualTo("value1");

			assertThat(fields.get(1).getName()).isEqualTo("key");
			assertThat(fields.get(1).getValue()).isEqualTo("value2");
		}
	}

	@Nested
	class TestMapObjectListsToFields {

		@Test
		void simpleValueShouldNotBeMapped() {

			List<GrpcSubForm> fields = MAPPER_INSTANCE.mapObjectListsToFields(Map.of("a", "b"));

			assertThat(fields).isEmpty();
		}

		@Test
		void listOfSimpleValueShouldNotBeMapped() {

			List<GrpcSubForm> fields = MAPPER_INSTANCE.mapObjectListsToFields(Map.of("a", List.of("l1", "l2")));

			assertThat(fields).isEmpty();
		}

		@Test
		void listOfObjectsShouldBeMappedToSubForms() {

			List<GrpcSubForm> fields = MAPPER_INSTANCE
					.mapObjectListsToFields(Map.of("key", List.of(Collections.emptyMap(), Collections.emptyMap())));

			assertThat(fields).hasSize(2);
		}

		@Test
		void listOfObjectsShouldMappedToSubFormTitle() {

			List<GrpcSubForm> fields = MAPPER_INSTANCE
					.mapObjectListsToFields(Map.of("key", List.of(Collections.emptyMap(), Collections.emptyMap())));

			assertThat(fields.get(0).getTitle()).isEqualTo("key");
			assertThat(fields.get(1).getTitle()).isEqualTo("key");
		}

		@Test
		void listOfObjectsShouldBeMapped() {

			List<GrpcSubForm> fields = MAPPER_INSTANCE
					.mapObjectListsToFields(Map.of("key", List.of(Map.of("a1", "a2"))));

			assertThat(fields.get(0).getFieldCount()).isEqualTo(1);
			assertThat(fields.get(0).getField(0).getName()).isEqualTo("a1");
			assertThat(fields.get(0).getField(0).getValue()).isEqualTo("a2");
		}

		@Test
		void doubleNestedListObjectValuesShouldBeMapped() {
			GrpcFormData formData = MAPPER_INSTANCE
					.mapToFormData(Map.of("key1", Map.of("key2", List.of("value1", "value2"))));

			assertThat(formData.getForm(0).getFieldCount()).isEqualTo(2);
		}

		@Test
		void multipleNestedListObjectValuesShouldBeMapped() {
			GrpcFormData formData = MAPPER_INSTANCE
					.mapToFormData(Map.of("key1", Map.of("key2", Map.of("key3", List.of("value1", "value2")))));

			assertThat(formData.getForm(0).getSubForm(0).getFieldCount()).isEqualTo(2);
		}
	}

	@Nested
	class TestMapListOfMixedValuesInFormData {

		@Test
		void shouldMapListOfStrings() {

			GrpcFormData formData = MAPPER_INSTANCE
					.mapToFormData(Map.of("key", List.of("value1", "value2", Map.of("internKey1", "internValue1"))));

			assertThat(formData.getFieldCount()).isEqualTo(2);
			assertThat(formData.getFormCount()).isEqualTo(1);
		}
	}
}
