/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.vorgang;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import java.io.File;
import java.io.FileOutputStream;

import org.apache.commons.io.FileUtils;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.EnumSource;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.mock.web.MockHttpServletResponse;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;

import de.ozgcloud.alfa.common.attacheditem.VorgangAttachedItemService;
import de.ozgcloud.alfa.common.command.CommandController;
import de.ozgcloud.alfa.common.user.CurrentUserService;
import de.ozgcloud.alfa.common.user.UserProfileTestFactory;
import de.ozgcloud.alfa.common.user.UserRole;
import de.ozgcloud.alfa.postfach.PostfachMailController;
import de.ozgcloud.alfa.vorgang.Vorgang.VorgangStatus;
import lombok.SneakyThrows;

@AutoConfigureMockMvc
@SpringBootTest
@WithMockUser
class VorgangITCase {

	@MockBean
	private VorgangRemoteService remoteService;

	@MockBean
	private CommandController commandController;

	@MockBean
	private CurrentUserService userService;

	@MockBean
	private PostfachMailController postfachController;

	@MockBean
	private VorgangAttachedItemService vorgangAttachedItemService;

	@Autowired
	private MockMvc mockMvc;

	@WithMockUser
	@Nested
	class TestGetVorgangWithEingang {

		static final String SINGLE_PATH = VorgangController.PATH + "/{id}";

		@BeforeEach
		void mockStub() {
			when(postfachController.isPostfachConfigured()).thenReturn(true);
			when(remoteService.findVorgangWithEingang(anyString(), any())).thenReturn(VorgangWithEingangTestFactory.create());
		}

		@Test
		void shouldReturnVorgangOnMatchingOrganisationseinheitId() throws Exception {
			when(userService.getUser()).thenReturn(
					UserProfileTestFactory.createBuilder().clearOrganisationseinheitIds()
							.organisationseinheitId(ZustaendigeStelleTestFactory.ORGANISATIONSEINHEITEN_ID).build());

			doRequest().andExpect(status().isOk());
		}

		@Test
		void shouldReturnVorgangOnEmptyUserOrganisationseinheitIdList() throws Exception {
			when(userService.getUser()).thenReturn(UserProfileTestFactory.createBuilder().clearOrganisationseinheitIds().build());

			doRequest().andExpect(status().isOk());
		}

		@Nested
		class TestAuthorizationByRole {

			@Nested
			class TestForVerwaltungPoststelle {

				@BeforeEach
				void mockUserService() {
					when(userService.hasRole(UserRole.VERWALTUNG_POSTSTELLE)).thenReturn(true);
				}

				@DisplayName("should return http forbidden status if vorgangstatus is different to 'Neu'")
				@ParameterizedTest
				@EnumSource(value = VorgangStatus.class, names = { "ANGENOMMEN", "VERWORFEN", "IN_BEARBEITUNG", "BESCHIEDEN", "ABGESCHLOSSEN",
						"WEITERGELEITET" })
				void shouldReturnForbiddenStatus(VorgangStatus status) throws Exception {
					when(remoteService.findVorgangWithEingang(anyString(), any()))
							.thenReturn(VorgangWithEingangTestFactory.createBuilder().status(status).build());

					doRequest().andExpect(status().isForbidden());
				}

				@DisplayName("should return http ok status if vorgangstatus is equal 'Neu'")
				@Test
				void shouldReturnOkStatus() throws Exception {
					when(remoteService.findVorgangWithEingang(anyString(), any()))
							.thenReturn(VorgangWithEingangTestFactory.createBuilder().status(VorgangStatus.NEU).build());

					doRequest().andExpect(status().isOk());
				}
			}

			@Nested
			class TestForOtherRoles {

				@BeforeEach
				void mockUserRole() {
					when(userService.hasRole(UserRole.VERWALTUNG_POSTSTELLE)).thenReturn(false);
					when(userService.hasRole(UserRole.EINHEITLICHER_ANSPRECHPARTNER)).thenReturn(false);
					when(userService.getUser()).thenReturn(UserProfileTestFactory.create());
				}

				@DisplayName("should return true on role 'VERWALTUNG_USER' independent which vorgangstatus")
				@ParameterizedTest
				@EnumSource(value = VorgangStatus.class, names = { "NEU", "ANGENOMMEN", "VERWORFEN", "IN_BEARBEITUNG", "BESCHIEDEN", "ABGESCHLOSSEN",
						"WEITERGELEITET" })
				void shouldReturnTrue(VorgangStatus status) throws Exception {
					when(remoteService.findVorgangWithEingang(anyString(), any()))
							.thenReturn(VorgangWithEingangTestFactory.createBuilder().status(status).build());

					doRequest().andExpect(status().isOk());
				}
			}
		}

		private ResultActions doRequest() throws Exception {
			return mockMvc.perform(get(SINGLE_PATH, VorgangHeaderTestFactory.ID));
		}
	}

	@Disabled("Der Test schreibt die Datei auf die Platte.")
	@Nested
	class TestExport {

		@SneakyThrows
		@Test
		void shouldCreateZip() {
			when(remoteService.findVorgangWithEingang(any(), any())).thenReturn(VorgangWithEingangTestFactory.create());
			when(userService.getUser()).thenReturn(UserProfileTestFactory.create());

			var response = doRequest().andReturn().getResponse();
			var content = response.getContentAsByteArray();
			var fileName = getFileName(response);

			writeFile(content, fileName);

			var wroteableFile = FileUtils.getFile(fileName);
			assertThat(wroteableFile).isNotNull();
		}

		private String getFileName(MockHttpServletResponse response) {
			var header = response.getHeaders("Content-Disposition").get(0);
			return header.substring(header.indexOf("filename=") + 9);
		}

		@SneakyThrows
		private void writeFile(byte[] content, String fileName) {
			var file = new File(fileName);
			var fos = new FileOutputStream(file);
			fos.write(content);
			fos.close();
		}

		@SneakyThrows
		private ResultActions doRequest() throws Exception {
			return mockMvc.perform(asyncDispatch(
					mockMvc.perform(get(VorgangController.PATH + "/" + VorgangHeaderTestFactory.ID).header(HttpHeaders.ACCEPT,
							MediaType.APPLICATION_OCTET_STREAM_VALUE)).andReturn()))
					.andExpect(status().isOk());
		}
	}
}