package de.ozgcloud.alfa.export;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.nio.charset.StandardCharsets;
import java.util.stream.Stream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.itvsh.kop.common.binaryfile.TempFileUtils;
import de.itvsh.kop.common.errorhandling.TechnicalException;
import de.ozgcloud.alfa.common.file.OzgFile;
import de.ozgcloud.alfa.vorgang.VorgangController;
import de.ozgcloud.alfa.vorgang.VorgangWithEingang;

@Service
class XDomeaService {

	static final String EXPORT_FILENAME_SUFFIX = "_Abgabe.Abgabe.0401.xml";

	private static final String EXPORT_FILENAME_TEMPLATE = "%s" + EXPORT_FILENAME_SUFFIX;

	@Autowired
	private XDomeaXmlMarshaller xDomeaXmlMarshaller;

	@Autowired
	private VorgangController vorgangController;

	@Autowired
	private AbgabeAbgabe0401Creator abgabeCreator;

	@Autowired
	private ExportFileService exportFileService;

	@Autowired
	private ExportFilenameGenerator exportFilenameGenerator;

	public void writeExport(String vorgangId, String filenameId, OutputStream out) {
		var exportData = collectExportData(vorgangId, filenameId);
		var zipFile = createZipFile(exportData);
		writeZipFileContent(zipFile, out);
	}

	ExportData collectExportData(String vorgangId, String filenameId) {
		var vorgang = vorgangController.getVorgang(vorgangId);
		var xmlContent = createXmlContent(vorgang);
		var filename = buildXmlFilename(filenameId);
		var ozgFiles = exportFileService.getAllPdfs(vorgang.getEingang().getId());
		return ExportData.builder().vorgang(vorgang).xmlFileContent(xmlContent).exportFilename(filename).exportFiles(ozgFiles).build();
	}

	String buildXmlFilename(String filenameId) {
		return String.format(EXPORT_FILENAME_TEMPLATE, filenameId);
	}

	String createXmlContent(VorgangWithEingang vorgang) {
		return xDomeaXmlMarshaller.marshal(abgabeCreator.create(vorgang));
	}

	File createZipFile(ExportData exportData) {
		var file = TempFileUtils.createTmpFile().toFile();
		try (var zipOutputStream = new ZipOutputStream(new FileOutputStream(file))) {
			putZipEntry(exportData.getExportFilename(), exportData.getXmlFileContent(), zipOutputStream);
			putFilesIntoZip(exportData.getExportFiles(), zipOutputStream);
			return file;
		} catch (Exception e) {
			throw new TechnicalException("Error creating XDomea zip file", e);
		}
	}

	void putZipEntry(String fileName, String fileData, ZipOutputStream zipOutputStream) throws IOException {
		var entry = new ZipEntry(fileName);
		zipOutputStream.putNextEntry(entry);
		zipOutputStream.write(fileData.getBytes(StandardCharsets.UTF_8));
		zipOutputStream.closeEntry();
	}

	private void putFilesIntoZip(Stream<OzgFile> ozgFiles, ZipOutputStream zipOutputStream) throws IOException {
		for (var ozgFile : ozgFiles.toList()) {
			putOzgFileIntoZip(ozgFile, zipOutputStream);
		}
	}

	void putOzgFileIntoZip(OzgFile ozgFile, ZipOutputStream zipOutputStream) throws IOException {
		var entry = new ZipEntry(exportFilenameGenerator.generateExportFilename(ozgFile));
		zipOutputStream.putNextEntry(entry);
		exportFileService.writeOzgFile(ozgFile.getId(), zipOutputStream);
		zipOutputStream.closeEntry();
	}

	void writeZipFileContent(File file, OutputStream outputStream) {
		try (var fileInputStream = new FileInputStream(file)) {
			fileInputStream.transferTo(outputStream);
		} catch (Exception e) {
			throw new TechnicalException("Error writting XDomea zip file to output stream", e);
		}
	}
}
