/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { UserProfileIconE2EComponent } from '../../../components/user-profile/user-profile-icon.component.e2e';
import { UserProfileE2EComponent } from '../../../components/user-profile/user-profile.component.e2e';
import { VorgangListE2EComponent } from '../../../components/vorgang/vorgang-list.e2e.component';
import { HttpMethodE2E, MessageCodeE2E } from '../../../model/util';
import { VorgangE2E } from '../../../model/vorgang';
import { MainPage, waitForSpinnerToDisappear } from '../../../page-objects/main.po';
import { VorgangPage } from '../../../page-objects/vorgang.po';
import { hasTooltip } from '../../../support/angular.util';
import { dropCollections, interceptWithResponse, waitOfInterceptor } from '../../../support/cypress-helper';
import { exist } from '../../../support/cypress.util';
import { MessagesE2E } from '../../../support/messages';
import { getUserManagerUserEmil, getUserManagerUserPeter, getUserManagerUserSabine, initUsermanagerUsers, loginAsSabine } from '../../../support/user-util';
import { createVorgang, initVorgang } from '../../../support/vorgang-util';

describe('User profile assigned to vorgang', () => {
	const mainPage: MainPage = new MainPage();
	const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

	const vorgangPage: VorgangPage = new VorgangPage();
	const userProfileContainer: UserProfileE2EComponent = vorgangPage.getVorgangDetailHeader().getUserContainer();

	const vorgang: VorgangE2E = { ...createVorgang(), assignedTo: '6287e7507433b51bfc87cfe9' };

	before(() => {
		initVorgang(vorgang);
		initUsermanagerUsers([getUserManagerUserSabine(), getUserManagerUserPeter(), getUserManagerUserEmil()]);

		loginAsSabine();

		waitForSpinnerToDisappear();
		exist(vorgangList.getRoot());
	})

	after(() => {
		dropCollections();
	})

	describe('on response status 404', () => {

		describe('in vorgang detail', () => {

			it('should navigate to vorgang detail', () => {
				vorgangList.getListItem(vorgang.name).getRoot().click();
				waitForSpinnerToDisappear();

				exist(vorgangPage.getVorgangDetailHeader().getRoot());
			})

			it('should show icon', () => {
				exist(userProfileContainer.getIconContainer().getErrorResourceNotFoundIcon());
			})

			it('should have tooltip', () => {
				hasTooltip(userProfileContainer.getIconContainer().getErrorResourceNotFoundIcon(), MessagesE2E.USER_PROFILE_RESOURCE_NOT_FOUND);
			})

			it('should navigate back', () => {
				vorgangPage.getSubnavigation().getBackButton().click();

				waitForSpinnerToDisappear();
				exist(vorgangList.getRoot());
			})
		})

		describe('in vorgang list', () => {

			const userProfile: UserProfileIconE2EComponent = vorgangList.getListItem(vorgang.name).getUserProfile().getIconContainer();

			it('should show icon', () => {
				exist(userProfile.getErrorResourceNotFoundIcon());
			})

			it('should have tooltip', () => {
				hasTooltip(userProfile.getErrorResourceNotFoundIcon(), MessagesE2E.USER_PROFILE_RESOURCE_NOT_FOUND);
			})
		})
	})

	describe('on response status 503', () => {

		const statusCode: number = 503;
		const getUserProfileInterceptor: string = 'getUserProfileInterceptor';

		beforeEach(() => {
			interceptWithResponse(HttpMethodE2E.GET, '/api/userProfiles/*', { statusCode, body: { issues: [{ messageCode: MessageCodeE2E.SERVICE_UNAVAILABLE }] } }).as(getUserProfileInterceptor);
		})

		describe('in vorgang detail', () => {

			it('should manipulate response', () => {
				vorgangList.getListItem(vorgang.name).getRoot().click();

				waitOfInterceptor(getUserProfileInterceptor).then((interception) => {
					assert(interception.response.statusCode, statusCode.toString());
					exist(vorgangPage.getVorgangDetailHeader().getRoot());
					waitForSpinnerToDisappear();
				});
			})

			it('should show icon', () => {
				exist(userProfileContainer.getIconContainer().getErrorServiceUnavailableIcon());
			})

			it('should have tooltip', () => {
				hasTooltip(userProfileContainer.getIconContainer().getErrorServiceUnavailableIcon(), MessagesE2E.USER_PROFILE_SERVICE_UNAVAILABLE);
			})
		})

		describe('in vorgang list', () => {

			const userProfile: UserProfileIconE2EComponent = vorgangList.getListItem(vorgang.name).getUserProfile().getIconContainer();

			it('should navigate to list', () => {
				vorgangPage.getSubnavigation().getBackButton().click();

				waitOfInterceptor(getUserProfileInterceptor).then((interception) => {
					assert(interception.response.statusCode, statusCode.toString());
					waitForSpinnerToDisappear();
					exist(vorgangList.getRoot());
				});
			})

			it('should show icon', () => {
				exist(userProfile.getErrorServiceUnavailableIcon());
			})

			it('should have tooltip', () => {
				hasTooltip(userProfile.getErrorServiceUnavailableIcon(), MessagesE2E.USER_PROFILE_SERVICE_UNAVAILABLE);
			})
		})
	})
})