/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { TestBed } from '@angular/core/testing';
import { ApiErrorAction, createStateResource } from '@goofy-client/tech-shared';
import { VorgangService, VorgangWithEingangResource } from '@goofy-client/vorgang-shared';
import { provideMockActions } from '@ngrx/effects/testing';
import { Action } from '@ngrx/store';
import { provideMockStore } from '@ngrx/store/testing';
import { cold, hot } from 'jest-marbles';
import { createHistorieListResource } from 'libs/historie-shared/test/historie';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { Observable, of } from 'rxjs';
import { mock } from '../historie.util';
import * as HistorieActions from './historie.actions';
import { HistorieEffects } from './historie.effects';
import { HistorieListResource } from './historie.model';
import { HistorieService } from './historie.service';

describe('HistorieEffects', () => {
	let actions: Observable<Action>;
	let effects: HistorieEffects;

	const historieService = mock(HistorieService);
	const vorgangService = mock(VorgangService);

	beforeEach(() => {
		TestBed.configureTestingModule({
			providers: [
				HistorieEffects,
				provideMockActions(() => actions),
				provideMockStore(),
				{
					provide: HistorieService,
					useValue: historieService
				},
				{
					provide: VorgangService,
					useValue: vorgangService
				}
			]
		});
		effects = TestBed.inject(HistorieEffects);
	});

	describe('loadAll', () => {

		const vorgangWithEingang: VorgangWithEingangResource = createVorgangWithEingangResource();

		beforeEach(() => {
			vorgangService.getVorgangWithEingang.mockReturnValue(of(createStateResource(vorgangWithEingang)));
		})

		it('should call vorgang service', () => {
			actions = of(HistorieActions.loadHistorie());

			effects.loadAll.subscribe();

			expect(vorgangService.getVorgangWithEingang).toHaveBeenCalled();
		})

		it('should call historie service', () => {
			actions = of(HistorieActions.loadHistorie());

			effects.loadAll.subscribe();

			expect(historieService.loadAll).toHaveBeenCalledWith(vorgangWithEingang);
		})

		it('should dispatch loadHistorieSucess action', () => {
			const historieList: HistorieListResource = createHistorieListResource();
			historieService.loadAll.mockReturnValue(of(historieList));

			actions = hot('-a-|', { a: HistorieActions.loadHistorie() });

			const expected = hot('-a-|', { a: HistorieActions.loadHistorieSuccess({ historieList }) });
			expect(effects.loadAll).toBeObservable(expected);
		})

		it('should dispatch loadHistorieFailed action', () => {
			const apiError = 'an error occured';
			const errorResponse = cold('-#', {}, apiError);
			historieService.loadAll = jest.fn(() => errorResponse);

			const expected = cold('--c', { c: HistorieActions.loadHistorieFailed(<ApiErrorAction><unknown>{ apiError }) });
			actions = hot('-a', { a: HistorieActions.loadHistorie() });

			expect(effects.loadAll).toBeObservable(expected);
		})
	})
});
