/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { NgModule } from '@angular/core';
import { TestBed } from '@angular/core/testing';
import { CommandResource } from '@goofy-client/command-shared';
import { createEmptyStateResource, createStateResource, StateResource } from '@goofy-client/tech-shared';
import { useFromMock } from '@goofy-client/test-utils';
import { UserProfileResource, UserProfileService } from '@goofy-client/user-profile-shared';
import { VorgangService } from '@goofy-client/vorgang-shared';
import { EffectsModule } from '@ngrx/effects';
import { Store, StoreModule } from '@ngrx/store';
import { readFirst } from '@nrwl/angular/testing';
import { createCommandResource } from 'libs/command-shared/test/command';
import { createVorgangResource } from 'libs/vorgang-shared/test/vorgang';
import { of, Subject } from 'rxjs';
import { createUserProfileResource } from '../../../../user-profile-shared/test/user-profile';
import { createHistorieListResource } from '../../../test/historie';
import { mock } from '../historie.util';
import { HistorieListProps } from './historie.actions';
import { HistorieEffects } from './historie.effects';
import { HistorieFacade } from './historie.facade';
import { HistorieListResource } from './historie.model';
import { HISTORIE_FEATURE_KEY, HistoriePartialState, reducer } from './historie.reducer';
import { HistorieService } from './historie.service';

import * as HistorieActions from './historie.actions';

describe('HistorieFacade ITCase', () => {
	let facade: HistorieFacade;
	let store: Store<HistoriePartialState>;

	const historieService = mock(HistorieService);
	const vorgangService = mock(VorgangService);
	const userProfileService = mock(UserProfileService);

	beforeEach(() => {
		@NgModule({
			imports: [
				StoreModule.forFeature(HISTORIE_FEATURE_KEY, reducer),
				EffectsModule.forFeature([HistorieEffects]),
			],
			providers: [HistorieFacade,
				{
					provide: HistorieService,
					useValue: historieService
				},
				{
					provide: VorgangService,
					useValue: vorgangService
				},
				{
					provide: UserProfileService,
					useValue: userProfileService
				}
			]
		})
		class CustomFeatureModule { }

		@NgModule({
			imports: [
				StoreModule.forRoot({}),
				EffectsModule.forRoot([]),
				CustomFeatureModule,
			]
		})
		class RootModule { }
		TestBed.configureTestingModule({ imports: [RootModule] });

		store = TestBed.inject(Store);
		facade = TestBed.inject(HistorieFacade);
	});



	it('is initialized', () => {
		expect(facade).toBeTruthy();
	})

	describe('get historie', () => {

		beforeEach(() => {
			historieService.loadAll.mockReturnValue(of(createHistorieListResource()));
			vorgangService.getVorgangWithEingang.mockReturnValue(of(createVorgangResource()));
		})

		it('should return loading list on required loading resource', async () => {
			let stateList = await readFirst(facade.getHistorie());

			expect(stateList.resource).toBeNull();
			expect(stateList.loading).toEqual(true);
		})

		it('should return the loaded list', async () => {
			const historieList: HistorieListResource = <HistorieListResource>createHistorieListResource();
			let stateList = await readFirst(facade.getHistorie());
			expect(stateList.resource).toBeNull();
			expect(stateList.loading).toEqual(true);

			store.dispatch(HistorieActions.loadHistorieSuccess(<HistorieListProps>{ historieList }));

			stateList = await readFirst(facade.getHistorie());
			expect(stateList.resource).toBe(historieList);
			expect(stateList.loading).toBe(false);
		});
	})

	describe('reload historie', () => {

		it('should dispatch action', () => {
			(<any>store).dispatch = jest.fn();

			facade.reloadHistorie();

			expect(store.dispatch).toHaveBeenCalledWith(HistorieActions.reloadHistorie());
		})
	})

	describe('get assigned to', () => {

		const userProfile: UserProfileResource = createUserProfileResource();
		const command: CommandResource = createCommandResource();

		beforeEach(() => {
			userProfileService.getAssignedUserProfile.mockReturnValue(of(createStateResource(userProfile)));
		})

		it('should call userProfileService', async () => {
			facade.getAssignedTo(command);

			expect(userProfileService.getAssignedUserProfile).toHaveBeenCalledWith(command, 'assignedTo');
		})

		it('should return user profile', async () => {
			const assignedTo = await readFirst(facade.getAssignedTo(command));

			expect(assignedTo.resource).toBe(userProfile);
		})
	})
});

describe('HistorieFacade UnitTest', () => {
	let facade: HistorieFacade;
	let store;

	let selectionSubject: Subject<StateResource<HistorieListResource>>;

	const userProfileService = mock(UserProfileService);

	beforeEach(() => {
		store = mock(Store);

		selectionSubject = new Subject();

		store.select.mockReturnValue(selectionSubject);
		store.dispatch = jest.fn();

		facade = new HistorieFacade(useFromMock(<any>store), useFromMock(<any>userProfileService));
	})

	it('is initialized', () => {
		expect(facade).toBeTruthy();
	})

	describe('get historie', () => {

		it('should return selected value', (done) => {
			const historieListResource: StateResource<HistorieListResource> = createStateResource(createHistorieListResource());

			facade.getHistorie().subscribe(selectorValue => {
				expect(selectorValue).toBe(historieListResource);
				done();
			});

			selectionSubject.next(historieListResource);
		});

		it('should dispatch action if not loaded', (done) => {
			facade.getHistorie().subscribe(historieList => {
				expect(store.dispatch).toHaveBeenCalledWith(HistorieActions.loadHistorie());
				expect(historieList.loading).toBe(true);
				done();
			});

			selectionSubject.next(createEmptyStateResource());
			selectionSubject.next(createEmptyStateResource(true));
		});
	})

	describe('reload historie', () => {

		it('should dispatch action', () => {
			(<any>store).dispatch = jest.fn();

			facade.reloadHistorie();

			expect(store.dispatch).toHaveBeenCalledWith(HistorieActions.reloadHistorie());
		})
	})

	describe('get assigned to', () => {

		const userProfile: UserProfileResource = createUserProfileResource();
		const command: CommandResource = createCommandResource();

		beforeEach(() => {
			userProfileService.getAssignedUserProfile.mockReturnValue(of(createStateResource(userProfile)));
		})

		it('should call userProfileService', async () => {
			facade.getAssignedTo(command);

			expect(userProfileService.getAssignedUserProfile).toHaveBeenCalledWith(command, 'assignedTo');
		})

		it('should return user profile', async () => {
			const assignedTo = await readFirst(facade.getAssignedTo(command));

			expect(assignedTo.resource).toBe(userProfile);
		})
	})
});