import { HttpStatusCode } from '@angular/common/http';
import faker from '@faker-js/faker';
import { CommandResource } from '@goofy-client/command-shared';
import { NavigationService } from '@goofy-client/navigation-shared';
import { StateResource, createStateResource } from '@goofy-client/tech-shared';
import { AdditionalActions, VorgangService, VorgangWithEingangResource } from '@goofy-client/vorgang-shared';
import { createCommandResource } from 'libs/command-shared/test/command';
import { createLoeschAnforderungResource } from 'libs/loesch-anforderung-shared/test/loesch-anforderung';
import { createHttpErrorResponse } from 'libs/tech-shared/test/http';
import { Mock, mock, useFromMock } from 'libs/test-utils/src/lib/mocking';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { of } from 'rxjs';
import { LoeschAnforderungFacade } from './+state/loesch-anforderung.facade';
import { LoeschAnforderungResource } from './loesch-anforderung.model';
import { LoeschAnforderungService } from './loesch-anforderung.service';
import { createLoeschAnforderungZuruecknehmenCommand, createVorgangLoeschenCommand, createVorgangZumLoeschenMarkierenCommand } from './loesch-anforderung.util';

import * as CommandUtil from '../../../command-shared/src/lib/command.util';

describe('LoeschAnforderungService', () => {
	let service: LoeschAnforderungService;

	let facade: Mock<LoeschAnforderungFacade>;
	let vorgangService: Mock<VorgangService>;
	let navigationService: Mock<NavigationService>;

	beforeEach(() => {
		facade = mock(LoeschAnforderungFacade);
		vorgangService = mock(VorgangService);
		navigationService = mock(NavigationService);

		service = new LoeschAnforderungService(useFromMock(facade), useFromMock(vorgangService), useFromMock(navigationService));
	})

	describe('getLoeschenAnfordernCommand', () => {

		it('should call facade', () => {
			service.getLoeschenAnfordernCommand();

			expect(facade.getLoeschenAnfordernCommand).toHaveBeenCalled();
		})
	})

	describe('loeschenAnfordern', () => {

		const vorgangWithEingang: VorgangWithEingangResource = createVorgangWithEingangResource();

		it('should call facade', () => {
			service.loeschenAnfordern(vorgangWithEingang);

			expect(facade.loeschenAnfordern).toHaveBeenCalledWith(vorgangWithEingang, createVorgangZumLoeschenMarkierenCommand());
		})
	})

	describe('getLoeschAnforderung', () => {

		it('should call facade', () => {
			service.getLoeschAnforderung();

			expect(facade.getLoeschAnforderung).toHaveBeenCalled();
		})
	})

	describe('getEndgueltigLoeschenCommand', () => {

		const command: CommandResource = createCommandResource();

		beforeEach(() => {
			facade.getEndgueltigLoeschenCommand.mockReturnValue(of(createStateResource(command)));
		})

		it('should call facade', () => {
			service.getEndgueltigLoeschenCommand().subscribe();

			expect(facade.getEndgueltigLoeschenCommand).toHaveBeenCalled();
		})

		it('should call vorgangService to get backButtonUrl', () => {
			service.getEndgueltigLoeschenCommand().subscribe();

			expect(vorgangService.getBackButtonUrl).toHaveBeenCalled();
		})

		it('should handleEndgueltigLoeschenCommandChanges', () => {
			const backButtonUrl: string = faker.internet.url();
			service.handleEndgueltigLoeschenCommandChanges = jest.fn();
			vorgangService.getBackButtonUrl.mockReturnValue(of(backButtonUrl));

			service.getEndgueltigLoeschenCommand().subscribe();

			expect(service.handleEndgueltigLoeschenCommandChanges).toHaveBeenCalledWith(command, backButtonUrl);
		})
	})

	describe('handleEndgueltigLoeschenCommandChanges', () => {

		const backButtonUrl: string = faker.internet.url();
		const command: CommandResource = createCommandResource();
		const spyOnCommandUtil = jest.spyOn(CommandUtil, 'isSuccessfulDone')

		it('should call isDoneWithoutError', () => {
			service.handleEndgueltigLoeschenCommandChanges(command, backButtonUrl);

			expect(spyOnCommandUtil).toHaveBeenCalledWith(command);
		})

		it('should call navigationService if command is done without error', () => {
			spyOnCommandUtil.mockReturnValue(true);

			service.handleEndgueltigLoeschenCommandChanges(command, backButtonUrl);

			expect(navigationService.navigate).toHaveBeenCalledWith(backButtonUrl);
		})

		it('should not navigate on error command', () => {
			spyOnCommandUtil.mockReturnValue(false);

			expect(navigationService.navigate).not.toHaveBeenCalled();
		})
	})

	describe('endgueltigLoeschen', () => {
		const loeschAnforderung: LoeschAnforderungResource = createLoeschAnforderungResource();

		it('should call facade', () => {
			service.endgueltigLoeschen(loeschAnforderung);

			expect(facade.endgueltigLoeschen).toHaveBeenCalledWith(loeschAnforderung, createVorgangLoeschenCommand());
		})
	})

	describe('getLoschAnforderungZuruecknehmenCommand', () => {

		const backButtonUrl: string = faker.internet.url();
		const command: CommandResource = createCommandResource();
		const commandStateResource: StateResource<CommandResource> = createStateResource(command);

		beforeEach(() => {
			facade.getLoeschAnforderungZuruecknehmenCommand.mockReturnValue(of(commandStateResource));
			vorgangService.getBackButtonUrl.mockReturnValue(of(backButtonUrl));
		})

		it('should call facade', () => {
			service.getLoschAnforderungZuruecknehmenCommand();

			expect(facade.getLoeschAnforderungZuruecknehmenCommand).toHaveBeenCalled();
		})

		it('should get back button Url', () => {
			service.getLoschAnforderungZuruecknehmenCommand();

			expect(vorgangService.getBackButtonUrl).toHaveBeenCalled();
		})

		it('should handleLoeschAnforderungZuruecknehmenCommandChanges', () => {
			service.handleLoeschAnforderungZuruecknehmenCommandChanges = jest.fn();

			service.getLoschAnforderungZuruecknehmenCommand().subscribe();

			expect(service.handleLoeschAnforderungZuruecknehmenCommandChanges).toHaveBeenCalledWith(commandStateResource, backButtonUrl);
		})

		it('should return command', (done) => {
			service.getLoschAnforderungZuruecknehmenCommand().subscribe(command => {
				expect(command).toBe(commandStateResource);
				done();
			});
		})
	})

	describe('handleLoeschAnforderungZuruecknehmenCommandChanges', () => {

		const overviewUrl: string = faker.internet.url();
		const command: CommandResource = createCommandResource();
		const commandStateResource: StateResource<CommandResource> = { ...createStateResource(command), error: <any>{ ...createHttpErrorResponse(), status: HttpStatusCode.NotFound } };

		it('should reload current vorgang on not found error', () => {
			service.handleLoeschAnforderungZuruecknehmenCommandChanges(commandStateResource, overviewUrl);

			expect(vorgangService.reloadCurrentVorgangWithAddtionalActions).toHaveBeenCalled();
		})

		it('should create additionalAction', () => {
			service.createAdditionalActions = jest.fn();

			service.handleLoeschAnforderungZuruecknehmenCommandChanges(commandStateResource, overviewUrl);

			expect(service.createAdditionalActions).toHaveBeenCalledWith(overviewUrl);
		})

		it('should reload current vorgang on not found error with additional actions', () => {
			const additionalActions: AdditionalActions = <any>{};
			service.createAdditionalActions = jest.fn().mockReturnValue(additionalActions);

			service.handleLoeschAnforderungZuruecknehmenCommandChanges(commandStateResource, overviewUrl);

			expect(vorgangService.reloadCurrentVorgangWithAddtionalActions).toHaveBeenCalledWith(additionalActions);
		})

		describe('createAdditionalActions', () => {

			it('shouldContainsAdditionalSuccessAction', () => {
				const additionalActions = service.createAdditionalActions(overviewUrl);

				expect(additionalActions.additionalSuccessAction).not.toBeNull();
			})

			it('shouldContainsAdditionalFailureAction', () => {
				const additionalActions = service.createAdditionalActions(overviewUrl);

				expect(additionalActions.additionalFailureAction).not.toBeNull();
			})
		})
	})

	describe('loeschAnforderungZuruecknehmen', () => {

		it('should call facade', () => {
			const loeschAnforderung: LoeschAnforderungResource = createLoeschAnforderungResource();

			service.zuruecknehmen(loeschAnforderung);

			expect(facade.loeschAnforderungZuruecknehmen).toHaveBeenCalledWith(loeschAnforderung, createLoeschAnforderungZuruecknehmenCommand());
		})
	})
})
