/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { PostfachMailListResource, PostfachService } from '@goofy-client/postfach-shared';
import { HasLinkPipe, StateResource, createErrorStateResource, createStateResource } from '@goofy-client/tech-shared';
import { mock } from '@goofy-client/test-utils';
import { DialogService } from '@goofy-client/ui';
import { VorgangHeaderLinkRel, VorgangWithEingangResource } from '@goofy-client/vorgang-shared';
import { createCommandResource } from 'libs/command-shared/test/command';
import { createPostfachMailListResource } from 'libs/postfach-shared/test/postfach';
import { createApiError } from 'libs/tech-shared/test/error';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { PostfachMailButtonContainerComponent } from './postfach-mail-button-container.component';
import { PostfachMailButtonComponent } from './postfach-mail-button/postfach-mail-button.component';

describe('PostfachMailButtonContainerComponent', () => {
	let component: PostfachMailButtonContainerComponent;
	let fixture: ComponentFixture<PostfachMailButtonContainerComponent>;

	const postfachService = mock(PostfachService);
	const dialogService = mock(DialogService);

	const vorgang: VorgangWithEingangResource = createVorgangWithEingangResource();

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				PostfachMailButtonContainerComponent,
				HasLinkPipe,
				MockComponent(PostfachMailButtonComponent)
			],
			providers: [
				{
					provide: DialogService,
					useValue: dialogService
				},
				{
					provide: PostfachService,
					useValue: postfachService
				}
			]
		}).compileComponents();
	});

	beforeEach(() => {
		fixture = TestBed.createComponent(PostfachMailButtonContainerComponent);
		component = fixture.componentInstance;
		component.vorgang = vorgang;
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('vorgang changes', () => {

		const vorgang: VorgangWithEingangResource = createVorgangWithEingangResource([VorgangHeaderLinkRel.POSTFACH_MAILS]);

		beforeEach(() => {
			component.loadPendingSendPostfachMailCommand = jest.fn();
			component.loadPostfachMailListStateResource = jest.fn();
		})

		it('should call loadPendingSendPostfachMailCommand', () => {
			component.vorgang = vorgang;

			expect(component.loadPendingSendPostfachMailCommand).toHaveBeenCalled();
		});

		it('should call loadPostfachMailListStateResource', () => {
			component.vorgang = vorgang;

			expect(component.loadPostfachMailListStateResource).toHaveBeenCalled();
		})
	})

	describe('loadPendingSendPostfachMailCommand', () => {

		beforeEach(() => {
			component.closeDialog = jest.fn();
			postfachService.getPendingSendPostfachMailCommand.mockReturnValue(of(createStateResource(createCommandResource())));
		})

		it('should call postfach service', () => {
			component.loadPendingSendPostfachMailCommand();

			expect(postfachService.getPendingSendPostfachMailCommand).toHaveBeenCalled();
		})

		it('should call closeDialog', () => {
			component.loadPendingSendPostfachMailCommand();

			expect(postfachService.getPendingSendPostfachMailCommand).toHaveBeenCalled();
		})
	})

	describe('loadPostfachMailListStateResource', () => {

		const vorgang: VorgangWithEingangResource = createVorgangWithEingangResource([VorgangHeaderLinkRel.POSTFACH_MAILS]);

		beforeEach(() => {
			postfachService.getPostfachMailListByGivenVorgang.mockReturnValue(of({}));
		})

		it('should call postfach service', () => {
			component.vorgang = vorgang;

			component.loadPostfachMailListStateResource();

			expect(postfachService.getPostfachMailListByGivenVorgang).toHaveBeenCalledWith(vorgang);
		})
	})

	describe('open postfach mail dialog', () => {

		const postfachMailListStateResource: StateResource<PostfachMailListResource> = createStateResource(createPostfachMailListResource());

		it('should open new dialog with given data', () => {
			component.openPostfachMailDialog(postfachMailListStateResource);

			expect(dialogService.openFixed).toHaveBeenCalled();
		})
	})

	describe('closeDialog', () => {

		it('should be called if command has no error and dialogRef is not undefined and resource is loaded', () => {
			component.dialogRef = <any>{};

			component.closeDialog(createStateResource(createCommandResource()));

			expect(dialogService.closeAll).toHaveBeenCalled();
		})

		it('should not be called if command is not loaded', () => {
			dialogService.closeAll.mockClear();
			component.dialogRef = <any>{};

			component.closeDialog(createErrorStateResource(createApiError()));

			expect(dialogService.closeAll).not.toHaveBeenCalled();
		})
	})

	describe('getEmpfaenger', () => {

		it('should return antragsteller name', () => {
			const antragsteller = component.getEmpfaenger();

			expect(antragsteller).toEqual(`${vorgang.eingang.antragsteller.vorname} ${vorgang.eingang.antragsteller.nachname}`)
		});
	})
});
