/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { HttpErrorResponse } from '@angular/common/http';
import { ApiRootResource } from '@goofy-client/api-root-shared';
import { Command, CommandOrder, CommandResource } from '@goofy-client/command-shared';
import { StateResource, createStateResource } from '@goofy-client/tech-shared';
import { Mock, mock, useFromMock } from '@goofy-client/test-utils';
import { Store } from '@ngrx/store';
import { ResourceUri } from '@ngxp/rest';
import { createApiRootResource } from 'libs/api-root-shared/test/api-root';
import { createCommand, createCommandResource } from 'libs/command-shared/test/command';
import { createVorgangListResource, createVorgangResources, createVorgangStatistic, createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { Subject } from 'rxjs';
import { VorgangWithEingangLinkRel } from '../vorgang.linkrel';
import { AdditionalActions, CreateAssignUserCommand, VorgangFilter, VorgangListResource, VorgangResource, VorgangStatistic, VorgangView, VorgangWithEingangResource } from '../vorgang.model';
import { VorgangFacade } from './vorgang.facade';

import faker from '@faker-js/faker';

import * as CommandActions from '../../../../command-shared/src/lib/+state/command.actions';
import * as VorgangActions from './vorgang.actions';
import * as VorgangSelectors from './vorgang.selectors';

describe('VorgangFacade', () => {
	let facade: VorgangFacade;
	let store: Mock<Store>;

	let selectionSubject: Subject<any>;

	const SEARCH_STRING = faker.word.noun();

	beforeEach(() => {
		store = mock(Store);

		selectionSubject = new Subject();

		store.select.mockReturnValue(selectionSubject);
		store.dispatch = jest.fn();

		facade = new VorgangFacade(useFromMock(<any>store));
	})

	it('is initialized', () => {
		expect(facade).toBeTruthy();
	})

	//VorgangList
	describe('getVorgangList', () => {

		it('should return selected value', (done) => {
			const vorgaengeStateResource: StateResource<VorgangResource[]> = createStateResource(createVorgangResources());

			facade.getVorgangList().subscribe(vorgaenge => {
				expect(vorgaenge).toBe(vorgaengeStateResource);
				done();
			});

			selectionSubject.next(vorgaengeStateResource);
		});
	})

	describe('loadVorgangList', () => {

		it('should dispatch "loadVorgangList" action', () => {
			const apiRoot: ApiRootResource = createApiRootResource();

			facade.loadVorgangList(apiRoot, 'dummyLinkRel');

			expect(store.dispatch).toHaveBeenCalledWith(VorgangActions.loadVorgangList({ apiRoot, linkRel: 'dummyLinkRel' }));
		});
	})

	describe('getVorgaenge', () => {

		it('should return selected value', (done) => {
			const vorgaengeStateResource: StateResource<VorgangResource[]> = createStateResource(createVorgangResources());

			facade.getVorgaenge().subscribe(vorgaenge => {
				expect(vorgaenge).toBe(vorgaengeStateResource);
				done();
			});

			selectionSubject.next(vorgaengeStateResource);
		});
	})

	describe('getVorgangStatistic', () => {

		it('should return selected value', (done) => {
			const vorgangStatisticStateResource: StateResource<VorgangStatistic> = createStateResource(createVorgangStatistic());

			facade.getVorgangStatistic().subscribe(vorgaenge => {
				expect(vorgaenge).toBe(vorgangStatisticStateResource);
				done();
			});

			selectionSubject.next(vorgangStatisticStateResource);
		});
	})

	describe('loadNextPage', () => {

		it('should dispatch "loadNextPage" action', () => {
			facade.loadNextPage();

			expect(store.dispatch).toHaveBeenCalledWith(VorgangActions.loadNextPage());
		});
	})

	describe('getVorgangFilter', () => {

		it('should return selected value', (done) => {
			const vorgangFilter: VorgangFilter = VorgangFilter.ALLE;

			facade.getVorgangFilter().subscribe(view => {
				expect(view).toBe(vorgangFilter);
				done();
			});

			selectionSubject.next(vorgangFilter);
		});
	})

	describe('isVorgangFilterSelected', () => {

		const isSelected: boolean = true;
		const vorgangFilter: VorgangFilter = VorgangFilter.ALLE;

		it('should return selected value', (done) => {
			facade.isVorgangFilterSelected(vorgangFilter).subscribe(selected => {
				expect(selected).toBe(isSelected);
				done();
			});

			selectionSubject.next(isSelected);
		})

		it('should call selector', (done) => {
			const selectorSpy = jest.spyOn(VorgangSelectors, 'isVorgangFilterSelected');

			facade.isVorgangFilterSelected(vorgangFilter).subscribe(() => {
				expect(selectorSpy).toHaveBeenCalledWith(vorgangFilter);
				done();
			});

			selectionSubject.next(isSelected);
		})
	})

	describe('getVorgangView', () => {

		it('should return selected value', (done) => {
			const vorgangView: VorgangView = VorgangView.NEU;

			facade.getVorgangView().subscribe(view => {
				expect(view).toBe(vorgangView);
				done();
			});

			selectionSubject.next(vorgangView);
		});
	})

	describe('isVorgangViewSelected', () => {

		const isSelected: boolean = true;
		const vorgangView: VorgangView = VorgangView.NEU;

		it('should return selected value', (done) => {
			facade.isVorgangViewSelected(vorgangView).subscribe(selected => {
				expect(selected).toBe(isSelected);
				done();
			});

			selectionSubject.next(isSelected);
		})

		it('should call selector', (done) => {
			const selectorSpy = jest.spyOn(VorgangSelectors, 'isVorgangViewSelected');

			facade.isVorgangViewSelected(vorgangView).subscribe(() => {
				expect(selectorSpy).toHaveBeenCalledWith(vorgangView);
				done();
			});

			selectionSubject.next(isSelected);
		})
	})

	describe('getVorgangViewRoutePath', () => {

		const path: string = 'dummyPath';
		const vorgangView: VorgangView = VorgangView.NEU;

		it('should return selected value', (done) => {
			facade.getVorgangViewRoutePath(vorgangView).subscribe(routePath => {
				expect(routePath).toBe(path);
				done();
			})

			selectionSubject.next(path);
		})

		it('should call selector',  (done) => {
			const selectorSpy = jest.spyOn(VorgangSelectors, 'getVorgangViewRoutePath');

			facade.getVorgangViewRoutePath(vorgangView).subscribe(() => {
				expect(selectorSpy).toHaveBeenCalledWith(vorgangView);
				done();
			})

			selectionSubject.next(path);
		})
	})

	describe('getBackButtonUrl', () => {

		it('should return selected value', (done) => {
			const path: string = 'dummyPath';

			facade.getBackButtonUrl().subscribe(routePath => {
				expect(routePath).toBe(path);
				done();
			})

			selectionSubject.next(path);
		})
	})

	//Search
	describe('searchVorgaengeBy', () => {

		const apiRoot: ApiRootResource = createApiRootResource();
		const searchString: string = SEARCH_STRING;
		const linkRel: string = 'LinkRelationName';

		it('should dispatch "searchVorgaengeBy" action', () => {
			facade.searchVorgaengeBy(apiRoot, searchString, linkRel);

			expect(store.dispatch).toHaveBeenCalledWith(VorgangActions.searchVorgaengeBy({ apiRoot, searchString, linkRel }));
		});
	})

	describe('searchForPreview', () => {

		it('should dispatch "searchForPreview" action', () => {
			facade.searchForPreview(SEARCH_STRING);

			expect(store.dispatch).toHaveBeenCalledWith(VorgangActions.searchForPreview({ string: SEARCH_STRING }));
		});
	})

	describe('setSearchString', () => {

		it('should dispatch "setSearchString" action', () => {
			const searchString: string = 'dummySearchString';

			facade.setSearchString(searchString);

			expect(store.dispatch).toHaveBeenCalledWith(VorgangActions.setSearchString({ string: searchString }));
		});
	})

	describe('getSearchString', () => {

		it('should return selected value', (done) => {
			const searchString: string = 'dummySearchString';

			facade.getSearchString().subscribe(selectedSearchString => {
				expect(selectedSearchString).toBe(searchString);
				done();
			});

			selectionSubject.next(searchString);
		});
	})

	describe('getSearchPreviewList', () => {

		it('should return selected value', (done) => {
			const saerchPreviewListStateResource: StateResource<VorgangListResource> = createStateResource(createVorgangListResource());

			facade.getSearchPreviewList().subscribe(searchPreviewList => {
				expect(searchPreviewList).toBe(saerchPreviewListStateResource);
				done();
			});

			selectionSubject.next(saerchPreviewListStateResource);
		});
	})

	describe('clearSearchPreviewList', () => {

		it('should dispatch "clearSearchPreviewList" action', () => {
			facade.clearSearchPreviewList();

			expect(store.dispatch).toHaveBeenCalledWith(VorgangActions.clearSearchPreviewList());
		});
	})

	describe('clearSearchString', () => {

		it('should dispatch "clearSearchString" action', () => {
			facade.clearSearchString();

			expect(store.dispatch).toHaveBeenCalledWith(VorgangActions.clearSearchString());
		})
	})

	//VorgangWithEingang
	describe('getVorgangWithEingang', () => {

		it('should return selected value', (done) => {
			const vorgangWithEingang: VorgangWithEingangResource = createVorgangWithEingangResource();

			facade.getVorgangWithEingang().subscribe(selectedVorgangWithEingang => {
				expect(selectedVorgangWithEingang).toBe(vorgangWithEingang);
				done();
			});

			selectionSubject.next(vorgangWithEingang);
		})
	})

	describe('loadVorgangWithEingang', () => {

		const vorgangWithEingangUri: ResourceUri = faker.internet.url();

		it('should dispatch "loadVorgangWithEingang" action', () => {
			facade.loadVorgangWithEingang(vorgangWithEingangUri);

			expect(store.dispatch).toHaveBeenCalledWith(VorgangActions.loadVorgangWithEingang({ resourceUri: vorgangWithEingangUri }));
		})
	})

	describe('loadVorgangWithEingangWithAdditionalActions', () => {

		const vorgangWithEingangUri: ResourceUri = faker.internet.url();

		it('should dispatch "loadVorgangWithEingang" action', () => {
			facade.loadVorgangWithEingang(vorgangWithEingangUri);

			expect(store.dispatch).toHaveBeenCalledWith(VorgangActions.loadVorgangWithEingang({ resourceUri: vorgangWithEingangUri }));
		})

		it('should dispatch "loadVorgangWithEingang" action with additional actions', () => {
			const additionalActions: AdditionalActions = { additionalSuccessAction: () => null, additionalFailureAction: (error: HttpErrorResponse) => null };

			facade.loadVorgangWithEingangWithAdditionalActions(vorgangWithEingangUri, additionalActions);

			expect(store.dispatch).toHaveBeenCalledWith(VorgangActions.loadVorgangWithEingang({ resourceUri: vorgangWithEingangUri, additionalActions }));
		})
	})


	describe('getAttachmentList', () => {

		it('should select from store', () => {
			facade.getAttachmentList();

			expect(store.select).toHaveBeenCalledWith(VorgangSelectors.attachmentList);
		})
	})

	describe('loadAttachmentList', () => {

		const vorgangWithEingang: VorgangWithEingangResource = createVorgangWithEingangResource();

		it('should dispatch "loadAttachmentList" action', () => {
			facade.loadAttachmentList(vorgangWithEingang);

			expect(store.dispatch).toHaveBeenCalledWith(VorgangActions.loadAttachmentList({ vorgangWithEingang }));
		})
	})

	describe('getRepresentationList', () => {

		it('should select from store', () => {
			facade.getRepresentationList();

			expect(store.select).toHaveBeenCalledWith(VorgangSelectors.representationList);
		})
	})

	describe('loadRepresentationList', () => {

		const vorgangWithEingang: VorgangWithEingangResource = createVorgangWithEingangResource();

		it('should dispatch "loadRepresentationList" action', () => {
			facade.loadRepresentationList(vorgangWithEingang);

			expect(store.dispatch).toHaveBeenCalledWith(VorgangActions.loadRepresentationList({ vorgangWithEingang }));
		})
	})

	describe('clearVorgangWithEingang', () => {

		it('should dispatch "clearnVorgangWithEingang" action', () => {
			facade.clearVorgangWithEingang();

			expect(store.dispatch).toHaveBeenCalledWith(VorgangActions.clearVorgangWithEingang());
		})
	})

	describe('getForwardPendingCommand', () => {

		it('should select from store', () => {
			facade.getForwardPendingCommand();

			expect(store.select).toHaveBeenCalledWith(VorgangSelectors.forwardPendingCommand);
		})
	})

	describe('setForwardSingleCommandLoading', () => {

		it('should dispatch "setForwardingSingleCommandLoading" action', () => {
			facade.setForwardSingleCommandLoading();

			expect(store.dispatch).toHaveBeenCalledWith(VorgangActions.setForwardingSingleCommandLoading());
		})
	})

	describe('setForwardSingleCommand', () => {

		const commandStateResource: StateResource<CommandResource> = createStateResource(createCommandResource());

		it('should dispatch "setForwardSingleCommand" action', () => {
			facade.setForwardSingleCommand(commandStateResource);

			expect(store.dispatch).toHaveBeenCalledWith(VorgangActions.setForwardingSingleCommand({ commandStateResource }));
		})
	})

	describe('getSendPostfachNachrichtPendingCommand', () => {

		it('should select from store', () => {
			facade.getSendPostfachNachrichtPendingCommand();

			expect(store.select).toHaveBeenCalledWith(VorgangSelectors.sendPostfachNachrichtPendingCommand);
		})
	})

	describe('setPendingSendPostfachMailSingleCommandLoading', () => {

		it('should dispatch "setSendPostfachNachrichtSingleCommandLoading" action', () => {
			facade.setPendingSendPostfachMailSingleCommandLoading();

			expect(store.dispatch).toHaveBeenCalledWith(VorgangActions.setSendPostfachNachrichtSingleCommandLoading());
		})
	})

	describe('setPendingSendPostfachMailSingleCommand', () => {

		const commandStateResource: StateResource<CommandResource> = createStateResource(createCommandResource());

		it('should dispatch "setSendPostfachNachrichtSingleCommand" action', () => {
			facade.setPendingSendPostfachMailSingleCommand(commandStateResource);

			expect(store.dispatch).toHaveBeenCalledWith(VorgangActions.setSendPostfachNachrichtSingleCommand({ commandStateResource }));
		})
	})

	describe('getAssignUserCommand', () => {

		it('should select from store', () => {
			facade.getAssignUserCommand();

			expect(store.select).toHaveBeenCalledWith(VorgangSelectors.assignUserCommand);
		})
	})

	describe('assignUser', () => {

		it('should dispatch createCommand action', () => {
			const vorgangWithEingang: VorgangWithEingangResource = createVorgangWithEingangResource();
			const linkRel: ResourceUri = faker.internet.url();
			const command : CreateAssignUserCommand = { order: CommandOrder.ASSIGN_USER, body: { assignedTo: linkRel } };

			facade.assignUser(vorgangWithEingang, command);

			expect(store.dispatch).toHaveBeenCalledWith(CommandActions.createCommand({ resource: vorgangWithEingang, linkRel: VorgangWithEingangLinkRel.ASSIGN, command }));
		})
	})

	describe('initAssignUser', () => {

		it('should dispatch initAssignUser action', () => {
			facade.initAssignUser();

			expect(store.dispatch).toHaveBeenCalledWith(VorgangActions.initAssignUser());
		})
	})

	describe('getAssignUserCommand', () => {

		it('should select from store', () => {
			facade.getAssignUserCommand();

			expect(store.select).toHaveBeenCalledWith(VorgangSelectors.assignUserCommand);
		})
	})

	describe('getStatusCommand', () => {

		const order: CommandOrder = CommandOrder.VORGANG_ANNEHMEN

		it('should select from store', (done) => {
			const selectorSpy = jest.spyOn(VorgangSelectors, 'getStatusCommand');

			facade.getStatusCommand(order).subscribe(() => {
				expect(selectorSpy).toHaveBeenCalledWith(order);
				done();
			});

			selectionSubject.next(createStateResource(createCommandResource()));
		})
	})

	describe('status actions', () => {

		const vorgang: VorgangWithEingangResource = createVorgangWithEingangResource();
		const command: Command = createCommand();

		describe('annehmen', () => {

			it('should dispatch action', () => {
				facade.annehmen(vorgang, command);

				expect(store.dispatch).toHaveBeenCalledWith(CommandActions.createCommand({ resource: vorgang, linkRel: VorgangWithEingangLinkRel.ANNEHMEN, command }));
			})
		})

		describe('bearbeiten', () => {

			it('should dispatch action', () => {
				facade.bearbeiten(vorgang, command);

				expect(store.dispatch).toHaveBeenCalledWith(CommandActions.createCommand({ resource: vorgang, linkRel: VorgangWithEingangLinkRel.BEARBEITEN, command }));
			})
		})

		describe('bescheiden', () => {

			it('should dispatch action', () => {
				facade.bescheiden(vorgang, command);

				expect(store.dispatch).toHaveBeenCalledWith(CommandActions.createCommand({ resource: vorgang, linkRel: VorgangWithEingangLinkRel.BESCHEIDEN, command }));
			})
		})

		describe('verwerfen', () => {

			it('should dispatch action', () => {
				facade.verwerfen(vorgang, command);

				expect(store.dispatch).toHaveBeenCalledWith(CommandActions.createCommand({ resource: vorgang, linkRel: VorgangWithEingangLinkRel.VERWERFEN, command }));
			})
		})

		describe('wiedereroeffnen', () => {

			it('should dispatch action', () => {
				facade.wiedereroeffnen(vorgang, command);

				expect(store.dispatch).toHaveBeenCalledWith(CommandActions.createCommand({ resource: vorgang, linkRel: VorgangWithEingangLinkRel.WIEDEREROEFFNEN, command }));
			})
		})

		describe('zurueckholen', () => {

			it('should dispatch action', () => {
				facade.zurueckholen(vorgang, command);

				expect(store.dispatch).toHaveBeenCalledWith(CommandActions.createCommand({ resource: vorgang, linkRel: VorgangWithEingangLinkRel.ZURUECKHOLEN, command }));
			})
		})

		describe('zurueckstellen', () => {

			it('should dispatch action', () => {
				facade.zurueckstellen(vorgang, command);

				expect(store.dispatch).toHaveBeenCalledWith(CommandActions.createCommand({ resource: vorgang, linkRel: VorgangWithEingangLinkRel.ZURUECKSTELLEN, command }));
			})
		})

		describe('abschliessen', () => {

			it('should dispatch action', () => {
				facade.abschliessen(vorgang, command);

				expect(store.dispatch).toHaveBeenCalledWith(CommandActions.createCommand({ resource: vorgang, linkRel: VorgangWithEingangLinkRel.ABSCHLIESSEN, command }));
			})
		})
	})

	describe('getRevokeCommand', () => {

		it('should select from store', () => {
			facade.getRevokeCommand();

			expect(store.select).toHaveBeenCalledWith(VorgangSelectors.revokeCommand);
		})
	})


	describe('getVorgangExport', () => {

		it('should select from store', () => {
			facade.getVorgangExport();

			expect(store.select).toHaveBeenCalledWith(VorgangSelectors.vorgangExport);
		})
	})

	describe('export', () => {

		const vorgangWithEingang: VorgangWithEingangResource = createVorgangWithEingangResource();

		it('should dispatch action', () => {
			facade.export(vorgangWithEingang);

			expect(store.dispatch).toHaveBeenCalledWith(VorgangActions.exportVorgang({ vorgangWithEingang }));
		})
	})
})