/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ScrollingModule } from '@angular/cdk/scrolling';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ApiRootLinkRel } from '@goofy-client/api-root-shared';
import { HintContainerComponent } from '@goofy-client/hint';
import { HasLinkPipe, StateResource, createEmptyStateResource, createStateResource } from '@goofy-client/tech-shared';
import { existsAsHtmlElement, getElementsFromFixture, mock, notExistsAsHtmlElement } from '@goofy-client/test-utils';
import { VorgangListLinkRel, VorgangListResource } from '@goofy-client/vorgang-shared';
import { createApiRootResource } from 'libs/api-root-shared/test/api-root';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { SpinnerComponent } from 'libs/ui/src/lib/ui/spinner/spinner.component';
import { createVorgangListResource, createVorgangResources } from 'libs/vorgang-shared/test/vorgang';
import { MockComponent } from 'ng-mocks';
import { EventEmitter } from 'stream';
import { EmptyListComponent } from './empty-list/empty-list.component';
import { VorgangListItemComponent } from './vorgang-list-item/vorgang-list-item.component';
import { VorgangListComponent } from './vorgang-list.component';

describe('VorgangListComponent', () => {
	let component: VorgangListComponent;
	let fixture: ComponentFixture<VorgangListComponent>;

	const vorgangListItem: string = 'goofy-client-vorgang-list-item';//TODO durch data-test-id ersetzen
	const hintContainer: string = getDataTestIdOf('hint-container');

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			imports: [
				ScrollingModule
			],
			declarations: [
				VorgangListComponent,
				HasLinkPipe,
				MockComponent(SpinnerComponent),
				MockComponent(VorgangListItemComponent),
				MockComponent(HintContainerComponent),
				MockComponent(EmptyListComponent)
			],
		}).compileComponents();
	});

	beforeEach(() => {
		fixture = TestBed.createComponent(VorgangListComponent);
		component = fixture.componentInstance;
		component.vorgangListPageResource = createEmptyStateResource();
		component.vorgaenge = createVorgangResources();
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	it('should have vorgang-list-item', () => {
		fixture.detectChanges();

		const elements = getElementsFromFixture(fixture, vorgangListItem);

		expect(elements).toHaveLength(10);
	});

	describe('onWindowsScroll', () => {

		beforeEach(() => {
			component.loadNextPage = jest.fn();
		})

		it('should call loadNextPage if scrolled to bottom', () => {
			component.isScrolledToBottom = jest.fn().mockReturnValue(true);

			component.onWindowScroll({});

			expect(component.loadNextPage).toHaveBeenCalled();
		})

		it('should do nothing if NOT scrolled to bottom', () => {
			component.isScrolledToBottom = jest.fn().mockReturnValue(false);

			component.onWindowScroll({});

			expect(component.loadNextPage).not.toHaveBeenCalled();
		})
	})

	describe('load next page', () => {

		beforeEach(() => {
			component.nextPage = <any>mock(EventEmitter);
		})

		it('should emit "nextPage" if necessary', () => {
			component.shouldLoadNextPage = jest.fn().mockReturnValue(true);

			component.loadNextPage();

			expect(component.nextPage.emit).toHaveBeenCalled();
		})

		it('should NOT emit "nextPage"', () => {
			component.shouldLoadNextPage = jest.fn().mockReturnValue(false);

			component.loadNextPage();

			expect(component.nextPage.emit).not.toHaveBeenCalled();
		})
	})

	describe('shouldLoadNextPage', () => {

		const vorgangListStateResource: StateResource<VorgangListResource> = createStateResource(createVorgangListResource([VorgangListLinkRel.NEXT]));

		it('should return true if resource is loaded', () => {
			component.vorgangListPageResource = vorgangListStateResource;

			const shouldLoadNextPage: boolean = component.shouldLoadNextPage();

			expect(shouldLoadNextPage).toBeTruthy();
		})

		it('should return false if resource is loading', () => {
			component.vorgangListPageResource = { ...vorgangListStateResource, loading: true };

			const shouldLoadNextPage: boolean = component.shouldLoadNextPage();

			expect(shouldLoadNextPage).toBeFalsy();
		})

		it('should return false if resource is reload required', () => {
			component.vorgangListPageResource = { ...vorgangListStateResource, reload: true };

			const shouldLoadNextPage: boolean = component.shouldLoadNextPage();

			expect(shouldLoadNextPage).toBeFalsy();
		})
	})

	describe('has next page', () => {

		it('should return true is link exists', () => {
			component.vorgangListPageResource = createStateResource(createVorgangListResource([VorgangListLinkRel.NEXT]));

			const hasNextPage: boolean = component.hasNextPage();

			expect(hasNextPage).toBeTruthy();
		})

		it('should return false if link NOT exists', () => {
			component.vorgangListPageResource = createStateResource(createVorgangListResource());

			const hasNextPage: boolean = component.hasNextPage();

			expect(hasNextPage).toBeFalsy();
		})
	})

	describe('hint container', () => {

		it('should show if link exists', () => {
			component.apiRootStateResource = createStateResource(createApiRootResource([ApiRootLinkRel.HINTS]));

			fixture.detectChanges();

			existsAsHtmlElement(fixture, hintContainer);
		})

		it('should hide if link NOT exists', () => {
			component.apiRootStateResource = createStateResource(createApiRootResource());

			fixture.detectChanges();

			notExistsAsHtmlElement(fixture, hintContainer);
		})
	})
});