/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { BinaryFileListResource, BinaryFileService } from '@goofy-client/binary-file-shared';
import { CommandOrder, CommandResource, CommandService } from '@goofy-client/command-shared';
import { NavigationService } from '@goofy-client/navigation-shared';
import { StateResource, createEmptyStateResource, createStateResource, decodeUrlFromEmbedding } from '@goofy-client/tech-shared';
import { Mock, mock, useFromMock } from '@goofy-client/test-utils';
import { SnackBarService } from '@goofy-client/ui';
import { VorgangResource, VorgangService, VorgangWithEingangResource } from '@goofy-client/vorgang-shared';
import { getUrl } from '@ngxp/rest';
import { cold, hot } from 'jest-marbles';
import { createBinaryFileListResource } from 'libs/binary-file-shared/test/binary-file';
import { CommandLinkRel } from 'libs/command-shared/src/lib/command.linkrel';
import { createCommandResource } from 'libs/command-shared/test/command';
import { createVorgangResource, createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { createWiedervorlage, createWiedervorlageListResource, createWiedervorlageResource } from 'libs/wiedervorlage-shared/test/wiedervorlage';
import { of } from 'rxjs';
import { WiedervorlageLinkRel, WiedervorlageListLinkRel } from './wiedervorlage.linkrel';
import { WiedervorlageMessages } from './wiedervorlage.message';
import { Wiedervorlage, WiedervorlageListResource, WiedervorlageResource } from './wiedervorlage.model';
import { WiedervorlageRepository } from './wiedervorlage.repository';
import { WiedervorlageService } from './wiedervorlage.service';

import * as TechShared from '@goofy-client/tech-shared';

describe('WiedervorlageService', () => {
	let service: WiedervorlageService;
	let repository: Mock<WiedervorlageRepository>;
	let navigationService: Mock<NavigationService>;
	let commandService: Mock<CommandService>;
	let snackbarService: Mock<SnackBarService>;
	let binaryFileService: Mock<BinaryFileService>;
	let vorgangService: Mock<VorgangService> = mock(VorgangService);

	const vorgangResource: VorgangWithEingangResource = createVorgangWithEingangResource();
	const wiedervorlageResource: WiedervorlageResource = createWiedervorlageResource();
	const wiedervorlage: Wiedervorlage = createWiedervorlage();
	const commandResource: CommandResource = createCommandResource();
	const commandResourceWithEffectedResourceLink: CommandResource = createCommandResource([CommandLinkRel.EFFECTED_RESOURCE]);

	beforeEach(() => {
		repository = mock(WiedervorlageRepository);
		navigationService = mock(NavigationService);
		commandService = mock(CommandService);
		snackbarService = mock(SnackBarService);
		binaryFileService = mock(BinaryFileService);
		vorgangService = mock(VorgangService);

		navigationService.urlChanged.mockReturnValue(of({}));

		service = new WiedervorlageService(
			useFromMock(repository),
			useFromMock(navigationService),
			useFromMock(commandService),
			useFromMock(snackbarService),
			useFromMock(binaryFileService),
			useFromMock(vorgangService)
		);
	})

	describe('getWiedervorlageList', () => {

		const wiedervorlageList: WiedervorlageListResource = createWiedervorlageListResource();
		const wiedervorlageListStateResource: StateResource<WiedervorlageListResource> = createStateResource(wiedervorlageList);

		const vorgangWithEingangStateResource: StateResource<VorgangWithEingangResource> = createStateResource(vorgangResource);

		beforeEach(() => {
			vorgangService.getVorgangWithEingang.mockReturnValue(of(vorgangWithEingangStateResource));
		})

		it('should call vorgangFacade getVorgangWithEingang', () => {
			service.getWiedervorlageList();

			expect(vorgangService.getVorgangWithEingang).toHaveBeenCalled();
		})

		it('should return values', () => {
			vorgangService.getVorgangWithEingang.mockReturnValue(hot('-a', { a: vorgangWithEingangStateResource }));
			service.wiedervorlageList$.asObservable = jest.fn().mockReturnValue(hot('-a', { a: wiedervorlageListStateResource }));

			const result = service.getWiedervorlageList();

			expect(result).toBeObservable(cold('ab', { a: createEmptyStateResource(true), b: wiedervorlageListStateResource }));
		})

		it.skip('should call loadWiedervorlagenByVorgang if required', (done) => {
			service.loadWiedervorlagenByVorgang = jest.fn();
			// service.wiedervorlageList$.next({ ...wiedervorlageListStateResource, reload: true });

			service.getWiedervorlageList().subscribe(list => {
				expect(service.loadWiedervorlagenByVorgang).toHaveBeenCalled();
					done();
				});
				// expect(service.loadWiedervorlagenByVorgang).toHaveBeenCalled();
				service.wiedervorlageList$.next({ ...wiedervorlageListStateResource, reload: true });
			// service.wiedervorlageList$.next({ ...wiedervorlageListStateResource, reload: true });
			service.getWiedervorlageList()
		})

		it('should NOT call loadWiedervorlagenByVorgang if already loaded', () => {
			service.loadWiedervorlagenByVorgang = jest.fn();
			service.wiedervorlageList$.asObservable = jest.fn().mockReturnValue(of(wiedervorlageListStateResource));

			service.getWiedervorlageList();

			expect(service.loadWiedervorlagenByVorgang).not.toHaveBeenCalled();
		})
	})

	describe('getWiedervorlageListByGivenVorgang', () => {

		const vorgang: VorgangResource = createVorgangResource();
		const wiedervorlageListResource: WiedervorlageListResource = createWiedervorlageListResource();
		const wiedervorlageListStateResource: StateResource<WiedervorlageListResource> = createStateResource(wiedervorlageListResource);

		it('should return initial values', () => {
			service.wiedervorlageList$.asObservable = jest.fn().mockReturnValue(hot('-a', { a: wiedervorlageListResource }));

			const result = service.getWiedervorlageListByGivenVorgang(vorgang);

			expect(result).toBeObservable(cold('ab', { a: createEmptyStateResource(true), b: wiedervorlageListResource }));
		})

		it('should load wiedervorlagen by vorgang if required', () => {
			service.loadWiedervorlagenByVorgang = jest.fn();
			service.wiedervorlageList$.asObservable = jest.fn().mockReturnValue(of(createEmptyStateResource()));

			service.getWiedervorlageListByGivenVorgang(vorgang).subscribe();

			expect(service.loadWiedervorlagenByVorgang).toHaveBeenCalledWith(vorgang);
		})

		it('should NOT load wiedervorlagen by vorgang if already loaded', () => {
			service.loadWiedervorlagenByVorgang = jest.fn();
			service.wiedervorlageList$.asObservable = jest.fn().mockReturnValue(of(wiedervorlageListStateResource));

			service.getWiedervorlageListByGivenVorgang(vorgang).subscribe();

			expect(service.loadWiedervorlagenByVorgang).not.toHaveBeenCalled();
		})
	})

	describe('getWiedervorlage', () => {

		const wiedervorlage: WiedervorlageResource = createWiedervorlageResource();
		const wiedervorlageStateResource: StateResource<WiedervorlageResource> = createStateResource(wiedervorlage);

		beforeEach(() => {
			navigationService.getDecodedParam.mockReturnValue(getUrl(wiedervorlageResource));
			repository.getWiedervorlage.mockReturnValue(of(wiedervorlageResource));
		})

		it('should return initial values', () => {
			service.wiedervorlage$.asObservable = jest.fn().mockReturnValue(hot('-a', { a: wiedervorlageStateResource }));

			const result = service.getWiedervorlage();

			expect(result).toBeObservable(cold('ab', { a: createEmptyStateResource(true), b: wiedervorlageStateResource }));
		})

		it('should set loading to true', () => {
			service.setWiedervorlageLoading = jest.fn();

			service.getWiedervorlage().subscribe();

			expect(service.setWiedervorlageLoading).toHaveBeenCalled();
		})

		it('should call navigationService', () => {
			service.getWiedervorlage().subscribe();

			expect(navigationService.getDecodedParam).toHaveBeenCalledWith(WiedervorlageService.encodedWiedervorlageUriParam);
		})

		it('should call repository', () => {
			service.getWiedervorlage().subscribe();

			expect(repository.getWiedervorlage).toHaveBeenCalledWith(getUrl(wiedervorlageResource));
		})

		it('should set returnvalue', () => {
			service.getWiedervorlage().subscribe();

			expect(service.wiedervorlage$.value).toEqual(createStateResource(wiedervorlageResource));
		})
	})

	describe('saveWiedervorlage', () => {

		beforeEach(() => {
			commandService.createCommand.mockReturnValue(of(createStateResource(commandResourceWithEffectedResourceLink)));
		})

		it('should return initial value and mapped value', () => {
			commandService.createCommand.mockReturnValue(hot('-a', { a: createStateResource(commandResourceWithEffectedResourceLink) }));

			const result = service.saveWiedervorlage(wiedervorlageResource, wiedervorlage);

			expect(result).toBeObservable(cold('ab', { a: createEmptyStateResource(true), b: createStateResource(commandResourceWithEffectedResourceLink) }));
		})

		it('should call command service', () => {
			service.saveWiedervorlage(wiedervorlageResource, wiedervorlage).subscribe();

			expect(commandService.createCommand).toHaveBeenCalledWith(wiedervorlageResource, WiedervorlageLinkRel.EDIT, { order: CommandOrder.EDIT_WIEDERVORLAGE, wiedervorlage, body: null });
		})

		it('should set submit-in-progress on loading', () => {
			(<any>service).setSubmitInProgressLoading = jest.fn();

			service.saveWiedervorlage(wiedervorlageResource, wiedervorlage);

			expect((<any>service).setSubmitInProgressLoading).toHaveBeenCalled();
		})

		it('should call proceedAfterRecieveCommand', () => {
			service.proceedAfterReceiveCommand = jest.fn();

			service.saveWiedervorlage(wiedervorlageResource, wiedervorlage).subscribe();

			const expectedMessage = WiedervorlageMessages.SAVED.replace('{betreff}', wiedervorlage.betreff);
			expect(service.proceedAfterReceiveCommand).toHaveBeenCalledWith(createStateResource(commandResourceWithEffectedResourceLink), expectedMessage);
		})
	})

	describe('createWiedervorlage', () => {

		beforeEach(() => {
			commandService.createCommand.mockReturnValue(of(createStateResource(commandResource)));
		})

		it('should return intitial value and mapped value', () => {
			commandService.createCommand.mockReturnValue(hot('-a', { a: createStateResource(commandResource) }));

			const result = service.createWiedervorlage(wiedervorlage);

			expect(result).toBeObservable(cold('ab', { a: createEmptyStateResource(true), b: createStateResource(commandResource) }));
		})

		it('should set submit-in-progress on loading', () => {
			(<any>service).setSubmitInProgressLoading = jest.fn();

			service.createWiedervorlage(wiedervorlage);

			expect((<any>service).setSubmitInProgressLoading).toHaveBeenCalled();
		})

		it('should call command service', () => {
			service.createWiedervorlage(wiedervorlage).subscribe();

			const command = { order: CommandOrder.CREATE_WIEDERVORLAGE, wiedervorlage, body: null };
			expect(commandService.createCommand).toHaveBeenCalledWith(service.wiedervorlageList$.value.resource, WiedervorlageListLinkRel.CREATE_WIEDERVORLAGE, command);
		})

		it('should call proceedAfterReceiveCommand', () => {
			service.proceedAfterReceiveCommand = jest.fn();

			service.createWiedervorlage(wiedervorlage).subscribe();

			const expectedMessage = WiedervorlageMessages.CREATED.replace('{betreff}', wiedervorlage.betreff);
			expect(service.proceedAfterReceiveCommand).toHaveBeenCalledWith(createStateResource(commandResource), expectedMessage);
		})
	})

	describe('proceedAfterRecieveCommand', () => {

		it('should set reload on wiedervorlageList', () => {
			service.proceedAfterReceiveCommand(createStateResource(commandResourceWithEffectedResourceLink), null);

			expect(service.wiedervorlageList$.value.reload).toBe(true);
		})

		it('should do nothing on apiError', () => {
			service.proceedAfterReceiveCommand(createEmptyStateResource(), null);

			expect(service.wiedervorlageList$.value.reload).toBe(false);
		})

		it('should call show snackbar', () => {
			snackbarService.show = jest.fn();
			const snackbarMessage: string = 'XX wurde XX';

			service.proceedAfterReceiveCommand(createStateResource(commandResourceWithEffectedResourceLink), snackbarMessage);

			expect(snackbarService.show).toHaveBeenCalledWith(commandResourceWithEffectedResourceLink, snackbarMessage);
		});
	})

	describe('order actions', () => {
		const wiedervorlageResource: WiedervorlageResource = createWiedervorlageResource();

		beforeEach(() => {
			commandService.createCommand.mockReturnValue(of(createStateResource(commandResourceWithEffectedResourceLink)));
			(<any>service).reloadWiedervorlageList = jest.fn();
			(<any>service).loadAndSetWiedervorlageByUrl = jest.fn();

			service.wiedervorlage$.next(createStateResource(wiedervorlageResource));
		})

		describe('wiedervorlage erledigen', () => {

			it('should call commandService', () => {
				service.erledigen();

				expect(commandService.createCommand).toHaveBeenCalledWith(wiedervorlageResource, WiedervorlageLinkRel.ERLEDIGEN, { order: CommandOrder.WIEDERVORLAGE_ERLEDIGEN, body: null });
			})

			it('should reload wiedervorlage', () => {
				service.erledigen();

				expect((<any>service).loadAndSetWiedervorlageByUrl).toHaveBeenCalled();
			})

			it('should show snackbar', () => {
				service.erledigen();

				expect(snackbarService.show).toHaveBeenCalledWith(commandResourceWithEffectedResourceLink, `Die Wiedervorlage ${wiedervorlageResource.betreff} wurde erledigt`);
			})
		})

		describe('wiedervorlage wiedereroeffnen', () => {

			it('should call commandService', () => {
				service.wiedereroeffnen();

				expect(commandService.createCommand).toHaveBeenCalledWith(wiedervorlageResource, WiedervorlageLinkRel.WIEDEREROEFFNEN, { order: CommandOrder.WIEDERVORLAGE_WIEDEREROEFFNEN, body: null });
			});

			it('should reload wiedervorlage', () => {
				service.wiedereroeffnen();

				expect((<any>service).loadAndSetWiedervorlageByUrl).toHaveBeenCalled();
			});

			it('should show snackbar', () => {
				service.wiedereroeffnen();

				expect(snackbarService.show).toHaveBeenCalledWith(commandResourceWithEffectedResourceLink, `Die Wiedervorlage ${wiedervorlageResource.betreff} wurde wiedereröffnet`);
			})
		})
	})

	describe('showSnackbar', () => {

		const message: string = 'SnackbarMessage';

		it('should call service if command is done', () => {
			const commandResource: CommandResource = createCommandResource([CommandLinkRel.EFFECTED_RESOURCE]);

			service.showSnackBar(commandResource, message);

			expect(snackbarService.show).toHaveBeenCalledWith(commandResource, message);
		})

		it('should NOT call service if command is pending', () => {
			const commandResource: CommandResource = createCommandResource();

			service.showSnackBar(commandResource, message);

			expect(snackbarService.show).not.toHaveBeenCalled();
		})
	})

	describe('on navigation', () => {

		beforeEach(() => {
			(<any>service).navigateToVorgangDetailPage = jest.fn();

			service.setWiedervorlageListReload = jest.fn();
			(<any>service).forceWiedervorlageReload = jest.fn();
			service.clearAttachmentList = jest.fn();
			service.clearWiedervorlagenList = jest.fn();
		})

		describe('to vorgang detail', () => {

			beforeEach(() => {
				(<any>service).navigateToVorgangDetailPage.mockReturnValue(true);
			})

			it('should set wiedervorlage list on reload', () => {
				service.onNavigation({});

				expect(service.setWiedervorlageListReload).toHaveBeenCalled();
			})

			it('should set wiedervorlage on reload', () => {
				service.onNavigation({});

				expect((<any>service).forceWiedervorlageReload).toHaveBeenCalled();
			})

			it('should clear attachments', () => {
				service.onNavigation({});

				expect(service.clearAttachmentList).toHaveBeenCalled();
			})
		})

		describe('to vorgang list', () => {

			it('should clear list', () => {
				service.onNavigation({});

				expect(service.clearWiedervorlagenList).toHaveBeenCalled();
			})

			it('should clear attachments', () => {
				service.onNavigation({});

				expect(service.clearAttachmentList).toHaveBeenCalled();
			})
		})
	})

	describe('load attachments', () => {

		const binaryFileListResource: BinaryFileListResource = createBinaryFileListResource();
		const binaryFileStateResource: StateResource<BinaryFileListResource> = createStateResource(binaryFileListResource);

		beforeEach(() => {
			binaryFileService.getFiles.mockReturnValue(of(binaryFileStateResource));
			service.setAttachmentLoading = jest.fn();
		})

		it('should set attachment state resource on loading', () => {
			service.loadAttachments(wiedervorlageResource);

			expect(service.setAttachmentLoading).toHaveBeenCalledWith(wiedervorlageResource);
		})

		it('should call file service', () => {
			service.loadAttachments(wiedervorlageResource);

			expect(binaryFileService.getFiles).toHaveBeenCalledWith(wiedervorlageResource, WiedervorlageLinkRel.ATTACHMENTS);
		})

		it('should set loaded resource into state resource', () => {
			service.loadAttachments(wiedervorlageResource);

			service.getAttachmentList(wiedervorlageResource);
			const result: StateResource<BinaryFileListResource> = service.attachmentListByWiedervorlage[getUrl(wiedervorlageResource)].value;

			expect(result).toEqual(binaryFileStateResource);
		})
	})

	describe('loadWiedervorlagenByVorgangStrict', () => {

		const wiedervorlageListResource: WiedervorlageListResource = createWiedervorlageListResource();

		beforeEach(() => {
			repository.getWiedervorlageList.mockReturnValue(hot('-a', { a: wiedervorlageListResource }));
		})

		it('should return value', () => {
			const result = service.loadWiedervorlagenByVorgangStrict(vorgangResource);

			expect(result).toBeObservable(cold('ab', { a: createEmptyStateResource(true), b: createStateResource(wiedervorlageListResource) }));
		})

		it('should call repository', () => {
			service.loadWiedervorlagenByVorgangStrict(vorgangResource);

			expect(repository.getWiedervorlageList).toHaveBeenCalledWith(vorgangResource);
		})
	})

	describe('hasEditLink', () => {

		beforeEach(() => {
			navigationService.getDecodedParam.mockReturnValue(getUrl(wiedervorlageResource));
			repository.getWiedervorlage.mockReturnValue(of(wiedervorlageResource));
		})

		it('should call decodeUrlFromEmbedding', () => {
			const spy = jest.spyOn(TechShared, 'decodeUrlFromEmbedding');

			service.hasEditLink(getUrl(wiedervorlageResource));

			expect(spy).toHaveBeenCalled();
		})

		it('should call repository.getWiedervorlage', () => {
			const url: string = getUrl(wiedervorlageResource);
			const decodedUrl: string = decodeUrlFromEmbedding(url);

			service.hasEditLink(url);

			expect(repository.getWiedervorlage).toHaveBeenCalledWith(decodedUrl);
		})

	})
})